/*
    i2c-MachZ.c - Linux kernel module for hardware monitoring
    Copyright (c) 2001 Hjalmar Hevers ,hhevers@ste-el.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the F ree Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
    This driver supports the ACCESS.BUS-Device on the MachZ from ZFLinux-Devices
*/

//#define TEST_IT

#include <linux/version.h>
#include <linux/module.h>
#include <asm/io.h>
#include <asm/ioctl.h>
#include <linux/kernel.h>
#include <linux/stddef.h>
#include <linux/sched.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <linux/vmalloc.h>


typedef long 						INT32;
typedef unsigned long 	UINT32;
typedef short 					INT16;
typedef unsigned short 	UINT16;
typedef char 						INT8;
typedef unsigned char 	UINT8;
typedef float						FLOAT32;
typedef double					FLOAT64;				

//#define CP42

#define EE_PFC8591_1	0x48
#define EE_PFC8591_2	0x49
#define EE_TCM1617 		0x4C

#ifdef CP42
#define EE_AT24C32		0x57

#else
#define EE_AT24C32		0x50
#endif


#define	NOTYPE					0
#define	AT24C32					1
#define	PFC8591					2
#define	TCM1617					3

/* MachZ-dependent Registers */
#define S_IO_BASEINDEXADR  0x2E
#define S_IO_BASEDATAADR   0x2F
#define S_IO_LOGICDEVICEOFFSET 0x07
#define S_IO_DEVICECONTROL 0x30
#define S_IO_DEVICEPORTADRESS  0x60
#define S_IO_DEVICEINTERRUPT   0x70
#define S_IO_DEVICEINTTYPE   0x71

#define S_IO_DEVENABLE  0x01
#define S_IO_DEVDISABLE  0
#define S_IO_SMBINTTYPE  0x02
#define S_IO_SMBINTERRUPT 0x03	// IRQ3 uk

#define SMB_LDN 0x08
#define SMB_BASE 0x300				// hier wuensche ich mein I/O_PORT
#define SMB_DATAOFFSET 0
#define SMB_STATUSOFFSET 1
#define SMB_CONSTATOFFSET 2
#define SMB_CONTROL1OFFSET 3
#define SMB_OWNADROFFSET 4
#define SMB_CONTROL2OFFSET 5

/* Other settings */
#define MAX_TIMEOUT 		50
#define SMB_MAJOR 			89
#define SMBSTATINIT 		1
#define SMBSTATOPEN 		2
#define SMBSTATBUSY 		3
#define SMBSTATREADY 		4
#define SMDWORDADRESS0 	5		// uk
#define SMDWORDADRESS1 	6   // uk



/* SMB_MachZ ioctl command constants */

long long i2c_lseek(struct file * file, long long offset, int origin);
int i2c_open(struct inode * inode, struct file * file);
int i2c_release (struct inode * inode, struct file * file);
ssize_t i2c_write(struct file * file,const char * buf, size_t count, loff_t *ppos);
ssize_t i2c_read(struct file * file,char * buf, size_t count, loff_t *ppos) ;
int i2c_ioctl(struct inode *inode, struct file *file,unsigned int cmd, unsigned long arg);



struct file_operations i2c_fops = {
llseek:		i2c_lseek,
write:	  i2c_write,
read: 	  i2c_read,
ioctl: 	  i2c_ioctl,
open:			i2c_open,
release:	i2c_release,
};


static struct smb_dev {
  UINT8 r_w_adress;		// device_address
  INT16 status;
  INT16 timeout;
  UINT32 count;
  UINT32 maxcount;
  UINT8 *datapointer;
  UINT8 wordaddress[2];	// uk
  INT16  rw_mode;				// uk
  INT16  mitadr ;				// uk
  INT16  opencnt;				// uk
  INT16  devtype;				// uk
} smb_device;

struct smb_channel {
  UINT8 adress;
  INT16 timeout;
};
  
/* insmod parameters */

static int force_addr = 0;
MODULE_PARM(force_addr, "i");
MODULE_PARM_DESC(force_addr,
		 "Forcibly enable the SMB_MachZ at the given address. ");
static int force_int = 0;
MODULE_PARM(force_int , "i");
MODULE_PARM_DESC(force_int ,
		 "Forcibly put the SMB_MachZ at the given Interrupt. ");
#ifdef MODULE
static
#else
extern
#endif
int __init i2c_SMB_MachZ_init(void);
static int __init SMB_MachZ_cleanup(void);
static int SMB_MachZ_setup(void);
void SmbIRQhd (int, void * ,struct pt_regs *);

#ifdef MODULE
extern int init_module(void);
extern int cleanup_module(void);
#endif				/* MODULE */


static unsigned short SMB_MachZ_smba = 0;
static unsigned short SMB_MachZ_int = 0;


int SMB_MachZ_setup(void)
{
	int error_return = 0;

#ifdef TEST_IT
	printk("i2c_adresse:%d\n",force_addr);
	printk("i2c_int:%d\n",force_int);
#endif

  if (force_addr)
		{
		SMB_MachZ_smba = force_addr & 0xfff0;
		}
  else
		{
		SMB_MachZ_smba = SMB_BASE;
		}

	if (force_int)
		{
		SMB_MachZ_int = force_int & 0x0F;
		}
  else
		{
	  SMB_MachZ_int =  S_IO_SMBINTERRUPT & 0x0F;
		}

	if (check_region(SMB_MachZ_smba, 6)) {
	  printk
	    ("i2c-SMB_MachZ.o: SMB_MachZ_smb region 0x%x already in use!\n",
	     SMB_MachZ_smba);
	  error_return = -ENODEV;
	  goto END;
	}
	if( request_irq(SMB_MachZ_int,SmbIRQhd,0,"SMB",NULL) !=0){
	  printk
	    ("i2c-SMB_MachZ.o: SMB_MachZ_int 0x%x already in use!\n",
	     SMB_MachZ_int);
	  error_return = -ENODEV;
	  goto END;
	}
	/* We program the new address here. Just to make
	   sure, we disable the SMB_MachZ first. */

	outb_p(   S_IO_LOGICDEVICEOFFSET, S_IO_BASEINDEXADR   );		// 0x07 nach Indexregister
	outb_p(   SMB_LDN ,  S_IO_BASEDATAADR  );										// 0x08 an 0x07 -> Accessbus current logic device
	outb_p(   S_IO_DEVICECONTROL, S_IO_BASEINDEXADR );   				// 0x30 nach Indexregister
	outb_p(   S_IO_DEVDISABLE,  S_IO_BASEDATAADR    );					// 0x00 nach 0x30 -> Device disable
	outb_p(   S_IO_DEVICEPORTADRESS+1, S_IO_BASEINDEXADR );     									// 0x61 nach Indexregister
	outb_p(   (unsigned char)(SMB_MachZ_smba & 0x0FF), S_IO_BASEDATAADR );  			// Base nach 0x61 -> AddressLimit 0..7
	outb_p(   S_IO_DEVICEPORTADRESS, S_IO_BASEINDEXADR );                   			// 0x60 nach Indexregister
	outb_p(   (unsigned char)((SMB_MachZ_smba>>8) & 0x0FF), S_IO_BASEDATAADR );   // Base nach 0x60 -> AddressLimit 8..15; default SMB_BASE 0x300
	outb_p(   S_IO_DEVICEINTERRUPT, S_IO_BASEINDEXADR );                          // 0x70 nach Indexregister
	outb_p(   (unsigned char)(SMB_MachZ_int&0x0FF), S_IO_BASEDATAADR );           // Int nach 0x70 -> InterruptSelect, default
	outb_p(   S_IO_DEVICEINTTYPE, S_IO_BASEINDEXADR );        // 0x70 nach Indexregister
	outb_p(   S_IO_SMBINTTYPE, S_IO_BASEDATAADR );            // 0x02 nach 0x70 -> Interrupt pos Flanke
	outb_p(   S_IO_DEVICECONTROL, S_IO_BASEINDEXADR );      	// 0x30 nach Indexregister
	outb_p(   S_IO_DEVENABLE, S_IO_BASEDATAADR);              // 0x01 nach 0x30 -> Device enable

	outb_p(0, SMB_MachZ_smba+SMB_OWNADROFFSET   );
	outb_p(((54<<1)+1), SMB_MachZ_smba+SMB_CONTROL2OFFSET   );
	request_region(SMB_MachZ_smba, 6,"SMB");
 END:
	return error_return;
}

int __init i2c_SMB_MachZ_init(void)
{
	int ret;
	if ((ret = SMB_MachZ_setup())) {
		printk
		    ("i2c-SMB_MachZ.o: SMB_MachZ-Module not inserted.\n");
		SMB_MachZ_cleanup();
		return ret;
	}
	printk("i2c-SMB_MachZ.o: SMB_MachZ bus initialized\n");
	if (register_chrdev(SMB_MAJOR,"smb",&i2c_fops)) {
		printk("i2c: unable to get major %d for i2c bus\n",SMB_MAJOR);
	}
	smb_device.status=SMBSTATINIT;
	smb_device.opencnt = 0;
	return 0;
}

int __init SMB_MachZ_cleanup(void)
{
  outb_p( (54<<1) ,SMB_MachZ_smba+SMB_CONTROL2OFFSET );//smb disable
  outb_p(   S_IO_LOGICDEVICEOFFSET, S_IO_BASEINDEXADR );
  outb_p(     SMB_LDN,  S_IO_BASEDATAADR );
  outb_p(   S_IO_DEVICECONTROL, S_IO_BASEINDEXADR );
  outb_p(     S_IO_DEVDISABLE, S_IO_BASEDATAADR );
  if (smb_device.status >=  SMBSTATINIT) {
    release_region(SMB_MachZ_smba, 6);
    free_irq(SMB_MachZ_int,NULL);
    smb_device.status =  0;
  }
  unregister_chrdev(SMB_MAJOR,"smb");
  return 0;
}

EXPORT_NO_SYMBOLS;

#ifdef MODULE

MODULE_AUTHOR
    ("Hjalmar Hevers > hhevers@ste-el.de");
MODULE_DESCRIPTION("SMB_MachZ SMBus driver");

int init_module(void)
{
	return i2c_SMB_MachZ_init();
}

int cleanup_module(void)
{
	return SMB_MachZ_cleanup();
}

#endif				/* MODULE */


void SmbIRQhd (int irq,void *dev_id ,struct pt_regs *reg_pnt)
{
unsigned char ee_read;
unsigned char ee_device_adr;

  if(inb_p(SMB_MachZ_smba+SMB_STATUSOFFSET)& 0x30)
		{//Buserror or Negack
    outb_p(0x0,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr disable
    outb_p(0x02,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr disable; Stop-Condition
    outb_p(0x70,SMB_MachZ_smba+SMB_STATUSOFFSET);
#ifdef TEST_IT
    printk("Buserror or Negack :%d\n",smb_device.status);
#endif
    smb_device.status=SMBSTATREADY;
    return;
  	}

  smb_device.timeout=0;
  switch (smb_device.status)
		{
/*  	case SMBSTATWAIT:
								{
     						smb_device.status =SMBSTATWAITTEST;
	 							outb_p(0x05,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr enable make startcondition
 							  ee_device_adr = smb_device.r_w_adress;
    						outb_p(ee_device_adr,SMB_MachZ_smba);//Adress_r_w - out
 								printk("in wait\n");
							  outb_p(0x03,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr enable; Stop-Condition
  							outb_p(0x05,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr enable make startcondition
								}
  	case SMBSTATWAITTEST:
  							if(inb_p(SMB_MachZ_smba+SMB_STATUSOFFSET)& 0x30)
									{
    							outb_p(0x0,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr disable
    							outb_p(0x02,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr disable; Stop-Condition
    							outb_p(0x70,SMB_MachZ_smba+SMB_STATUSOFFSET);
   								printk("Buserror or Negack :%d\n",smb_device.status);
      						smb_device.status =SMBSTATWAIT;
 	 								outb_p(0x05,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr enable make startcondition
									}
								else
	    						smb_device.status =SMBSTATOPEN;
                break;
*/
  	case SMBSTATOPEN:
							ee_device_adr = smb_device.r_w_adress;
							if(smb_device.devtype == AT24C32)
								{         // internes EEPROM mit erweiterter Adressierung
    				  	if(smb_device.rw_mode == 2)
	    						smb_device.status=SMBSTATBUSY;    // dummyread fertig, nur noch deviceadr und dann die daten lesen
								else
	    						smb_device.status=SMDWORDADRESS0;
    				  	if(smb_device.rw_mode == 1)
									ee_device_adr &= 0xFE;							// dummywrite!
								}
							else
               	smb_device.status=SMBSTATBUSY;
    					outb_p(ee_device_adr,SMB_MachZ_smba);//Adress_r_w - out
#ifdef TEST_IT
						  printk("i2c_adresse0 im I N T open:%d\n",ee_device_adr);
#endif
    					break;
    case SMDWORDADRESS0:
    					smb_device.status=SMDWORDADRESS1;
    					outb_p(smb_device.wordaddress[1],SMB_MachZ_smba);
#ifdef TEST_IT
						  printk("i2c_wordadresse1 im I N T open:%d\n",smb_device.wordaddress[1]);
#endif
    					break;
    case SMDWORDADRESS1:
    					outb_p(smb_device.wordaddress[0],SMB_MachZ_smba);
    				  if(smb_device.rw_mode)
								{
	    					smb_device.status=SMBSTATOPEN;
		            smb_device.rw_mode++;      	// dymmy-read erledift, nun weiter mit start_seq und device_adr
  							outb_p(0x05,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr enable make startcondition
								}
							else
    						smb_device.status=SMBSTATBUSY;
#ifdef TEST_IT
						  printk("i2c_wordadresse0 im I N T open:%d\n",smb_device.wordaddress[0]);
#endif
    					break;
   	case SMBSTATBUSY:
    				if(smb_device.rw_mode)
							{//read
      				if(smb_device.count >= smb_device.maxcount-1)
								{
								outb_p(0,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr disable
								outb_p(0x02,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr disable; Stop-Condition
								outb_p(0x70,SMB_MachZ_smba+SMB_STATUSOFFSET);	
								smb_device.status=SMBSTATREADY;
     			 			}
      					else if(smb_device.count >= smb_device.maxcount-2)
									{//last data
									outb_p(0x14,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//NACK
      						}
      				ee_read = *(smb_device.datapointer+smb_device.count++)=inb_p(SMB_MachZ_smba);
    					}
    				else
							{//write
      				if(smb_device.count>=smb_device.maxcount)
								{//last data
								outb_p(0x0,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr disable
								outb_p(0x02,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr disable; Stop-Condition
								outb_p(0x70,SMB_MachZ_smba+SMB_STATUSOFFSET);	
								smb_device.status=SMBSTATREADY;
//  							outb_p(0x01,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//make startcondition
								return;
      					}
							if(smb_device.mitadr)
								{		// jetzt brauche ich wieder eine neue Adresse
#ifdef TEST_IT
 						    printk("neue adresse at:%d\n", (*(INT16*)&smb_device.wordaddress[0]));
#endif
								outb_p(0x0,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr disable
								outb_p(0x02,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr disable; Stop-Condition
								outb_p(0x70,SMB_MachZ_smba+SMB_STATUSOFFSET);	
								smb_device.status=SMBSTATOPEN;          			// neue Adresse
								smb_device.mitadr = 0;
  							outb_p(0x04,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr enable
  							outb_p(0x05,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr enable make startcondition
								return;
								}
     		 			outb_p(*(smb_device.datapointer+smb_device.count++),SMB_MachZ_smba);//Data - out
							if(smb_device.devtype == AT24C32)
								{
						  	(*(INT16*)&smb_device.wordaddress[0])++;   // [0] wird hochgezaehlt!!
								if( (smb_device.wordaddress[0] & 0x1f) == 0)
									smb_device.mitadr = 1;	
                }
    					}
    				break;
  	default:
    				break;
 	}
  return;
}


int i2c_open(struct inode * inode, struct file * file)
{
struct smb_channel *channelp;


#ifdef TEST_IT
  printk("open :%d\n",smb_device.opencnt);
#endif
	if(smb_device.opencnt)
		return(-ENODEV);
	smb_device.opencnt++;
  channelp = vmalloc(sizeof(struct smb_channel));
  file->private_data = channelp;
  switch(MINOR(inode->i_rdev))
		{
  	case 0:
			smb_device.devtype = AT24C32;
    	channelp -> adress = EE_AT24C32;		// erstes device auf 1010 verdrahtet AT24C32
    	break;
  	case 1:
			smb_device.devtype = PFC8591;
     	channelp -> adress = EE_PFC8591_1;  // Netzteiladress Temp PFC8591
    	break;
  	case 2:
			smb_device.devtype = PFC8591;
     	channelp -> adress = EE_PFC8591_2;  // Netzteiladress Voltage PFC8591
    	break;
  	case 3:										 						// Temperatur ETX	
			smb_device.devtype = TCM1617;
    	channelp -> adress = EE_TCM1617;
    	break;
  	default:
    	channelp-> adress=0;
    	break;
  	}
  MOD_INC_USE_COUNT;
  return (0);
}

int i2c_release (struct inode * inode, struct file * file)
{
  vfree( file->private_data );
	smb_device.opencnt--;
  MOD_DEC_USE_COUNT;
  return(0);
}


long long i2c_lseek(struct file * file, long long offset, int origin)
{	
 if(origin == 0)
	{
 	*(INT16*)&smb_device.wordaddress[0] = offset;
  return(offset);
	}
return -ESPIPE;
}

ssize_t i2c_write(struct file * file,const char * buf, size_t count, loff_t *ppos)
{
unsigned char temp;

#ifdef TEST_IT
  printk("i2c_write\n");
  printk("count:%d\n",count);
#endif

  ((struct smb_channel *)file->private_data)->timeout=0;
  while(smb_device.status!=SMBSTATINIT && ((struct smb_channel *)file->private_data)->timeout++ < MAX_TIMEOUT)
		{
    current->state = TASK_INTERRUPTIBLE;
    schedule_timeout(1);
  	}
  if (((struct smb_channel *)file->private_data)->timeout >= MAX_TIMEOUT)
		return(0);
  smb_device.status = SMBSTATOPEN;	

  if(inb_p(SMB_MachZ_smba+ SMB_CONSTATOFFSET)&0x02)   // Bus active?
		outb_p(0x02,SMB_MachZ_smba+ SMB_CONSTATOFFSET);   // dann wohl irgenwie abwuergen? uk

  ((struct smb_channel *)file->private_data)->timeout=0;
  temp= inb_p(SMB_MachZ_smba + SMB_CONSTATOFFSET);
  while(( ((inb_p(SMB_MachZ_smba+ SMB_CONSTATOFFSET)&0x10)==0) ||
	 													((temp & 0x01)==1)) && ((struct smb_channel *)file->private_data)->timeout++ < MAX_TIMEOUT)
		{
    if((temp & 0x01)==1)
			{
      current->state = TASK_INTERRUPTIBLE;
      schedule_timeout(1);
      temp= inb_p(SMB_MachZ_smba + SMB_CONSTATOFFSET);
      if((temp & 1)==0)
				continue ;
    	}

    outb_p( (54<<1) ,SMB_MachZ_smba+SMB_CONTROL2OFFSET );//smb disable, hier steckt der clock mit drin uk 54: 100kHz
    outb_p( (54<<1)+1 ,SMB_MachZ_smba+SMB_CONTROL2OFFSET );//smb enable
    outb_p(0x22,SMB_MachZ_smba+ SMB_CONSTATOFFSET);
    current->state = TASK_INTERRUPTIBLE;
    schedule_timeout(1);
  	}
  if (((struct smb_channel *)file->private_data)->timeout >= MAX_TIMEOUT)
		{
    smb_device.status = SMBSTATINIT;
    return(0);
  	}
  outb_p(0x30,SMB_MachZ_smba+SMB_STATUSOFFSET);//all clear
  smb_device.r_w_adress = ((struct smb_channel *)file->private_data)->adress<<1;
  smb_device.rw_mode = 0;


  smb_device.datapointer=(unsigned char *)vmalloc(count);
  copy_from_user(smb_device.datapointer,buf,count);

  smb_device.timeout = 0;
  smb_device.count = 0;
  smb_device.mitadr = 0;

  smb_device.maxcount=count;
  outb_p(0x04,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr enable
  outb_p(0x05,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr enable make startcondition

  while( smb_device.status !=SMBSTATREADY && smb_device.timeout++ <  MAX_TIMEOUT)
		{
    current->state = TASK_INTERRUPTIBLE;
    schedule_timeout(1);
		}
  if(smb_device.timeout >=  MAX_TIMEOUT)
		{
    printk("Zeit abgelaufen\n");
    outb_p(0x02,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr disable; Stop-Condition
    outb_p(0x70,SMB_MachZ_smba+SMB_STATUSOFFSET);//all clear
		smb_device.count = -1; // uk
  	}

  vfree( smb_device.datapointer);
  smb_device.status =SMBSTATINIT;
  return(smb_device.count);
}

ssize_t i2c_read(struct file * file,char * buf, size_t count, loff_t *ppos)
{
unsigned char temp;


#ifdef TEST_IT
  printk("i2c_read\n");
#endif


  ((struct smb_channel *)file->private_data)->timeout=0;
  while(smb_device.status!=SMBSTATINIT && ((struct smb_channel *)file->private_data)->timeout++ < MAX_TIMEOUT)
		{
    current->state = TASK_INTERRUPTIBLE;
    schedule_timeout(1);
  	}
  if (((struct smb_channel *)file->private_data)->timeout >= MAX_TIMEOUT)
		return(0);

  smb_device.status = SMBSTATOPEN;
  if(inb_p(SMB_MachZ_smba+ SMB_CONSTATOFFSET)&0x02)
		outb_p(0x02,SMB_MachZ_smba+ SMB_CONSTATOFFSET);
  ((struct smb_channel *)file->private_data)->timeout=0;
  temp= inb_p(SMB_MachZ_smba + SMB_CONSTATOFFSET);
  while( ( ((inb_p(SMB_MachZ_smba+ SMB_CONSTATOFFSET)&0x10)==0) ||
							  ((temp & 0x01)==0x01)) && ((struct smb_channel *)file->private_data)->timeout++ < MAX_TIMEOUT)
		{
    if((temp & 0x01) == 0x01)
			{
      current->state = TASK_INTERRUPTIBLE;
      schedule_timeout(1);
      temp= inb_p(SMB_MachZ_smba + SMB_CONSTATOFFSET);
      if((temp & 0x01)==0)
				continue ;
    	}

    outb_p( (54<<1) ,SMB_MachZ_smba+SMB_CONTROL2OFFSET );//smb disable
    outb_p( (54<<1)+1 ,SMB_MachZ_smba+SMB_CONTROL2OFFSET );//smb enable
    outb_p(0x22,SMB_MachZ_smba+ SMB_CONSTATOFFSET);
    current->state = TASK_INTERRUPTIBLE;
    schedule_timeout(1);
  	}
  if (((struct smb_channel *)file->private_data)->timeout >= MAX_TIMEOUT)
		{
		printk("TIMEOUT READ1");
    smb_device.status = SMBSTATINIT;
    return(0);
  	}

  outb_p(0x70,SMB_MachZ_smba+SMB_STATUSOFFSET);//all Errors clear
  smb_device.r_w_adress=( ((struct smb_channel *)file->private_data)->adress<<1) + 1;
  smb_device.rw_mode = 1;

  smb_device.datapointer=(unsigned char *)vmalloc(count);
  smb_device.timeout=0;
  smb_device.count=0;
  smb_device.maxcount=count;
  outb_p(0x04,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr enable
  outb_p(0x05,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr enable make startcondition
  while( smb_device.status !=SMBSTATREADY && smb_device.timeout++ <  MAX_TIMEOUT)
		{
    current->state = TASK_INTERRUPTIBLE;
    schedule_timeout(1);
  	}
  if(smb_device.timeout >=  MAX_TIMEOUT)
		{
    outb_p(0x02,SMB_MachZ_smba+SMB_CONTROL1OFFSET);//interr disable; Stop-Condition
    outb_p(0x70,SMB_MachZ_smba+SMB_STATUSOFFSET);//all Errors clear
		printk("TIMEOUT READ2");
  	}
  copy_to_user(buf,smb_device.datapointer,smb_device.count);
  vfree( smb_device.datapointer);
  smb_device.status =SMBSTATINIT;
  return(smb_device.count);
}

int i2c_ioctl(struct inode *inode, struct file *file,unsigned int cmd, unsigned long arg)
{
  switch(cmd)
		{
  	case 0x654 /* SMB_MachZ_SETADRESS */ :
     					((struct smb_channel *)file->private_data)->adress=(unsigned char)(arg & 0x7F);
    					return(0);
  	case 0x655 /*  SMB_MachZ_GETREG */:
    					return(inb_p(SMB_MachZ_smba+((arg>>8) &0x0F)));
  	case 0x656 /*  SMB_MachZ_SETREG */:
    					outb_p((unsigned char)(arg&0x0FF) ,SMB_MachZ_smba+((arg>>8) &0x0F));
    					return(0);
  	default:
    					return(0);
  	}
}




