/***************************************************************************
                                  usbdev.h
                             -------------------
    project:  USB device for Philips PDIUSBD12D

    begin                : Dec 03, 2001
    copyright            : (C) 2001 by HBM GmbH, Siegfried Wessler
    email                : siegfried.wessler@hbm.com

    latest official version:  May 08, 2003

    updates:         May 08, 2003 Wessler: New Revision number

 ***************************************************************************/




/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef __USBDEV__
#define __USBDEV__    


// We use this driver in a non open system. So we can "allow" ourself to use
//   static Major etc numbers. Please be aware that you keep an eye on this,
//   if you use this driver on a public system.


#define MODULE_NAME         "usbdev"
#define MODULE_INFO_AUTHOR  "Siegfried Wessler, T-V, HBM GmbH"

#ifdef CP22
#define MODULE_INFO_DESC    "Driver for HBM CP22 for access as USB device via Philips PDIUSB12D"
#endif
#ifdef CP42
#define MODULE_INFO_DESC    "Driver for HBM CP42 for access as USB device via Philips PDIUSB12D"
#endif

#define DEV_MAJOR     196   // unused in "/lib/modules/2.4.4-4GB/build/Documentation/devices.txt"
                            // used in Kernel 2.4.18 by Tormenta T1
                            // @@@ check before publishing!


// In my "file:/lib/modules/2.4.4-4GB/build/Documentation/ioctl-number.txt"
// magic no. 0x9F is not mentioned. I try using this static. @@@ check before publishing!
// Be aware that Linux on CP22 will be in a close environment, so strange magics should
// usually not make any problem or at least can be handled by the developers of CP22,
// who should always know, who's modules are running in this embedded system.
#define USB_IOC_MAGIC       0x9F

#define USB_IOC_SET_ADDR  _IOW(USB_IOC_MAGIC, 1, 16)
#define USB_IOC_MAX_NR          1


#define USB_VENDOR_ID         0x10D1    // HBM unique vendor id
#define USB_SPEC_ID           0x0110    // USB 1.10 compliant
//#define USB_SPEC_ID           0x0200    // USB 2.00 compliant

#ifdef CP22
  #define USB_PRODUCT_ID      0x0201    // CP22
  #define USB_REVISION_ID     0x0103
#endif

#ifdef CP42
  #define USB_PRODUCT_ID      0x0202    // CP42
  #define USB_REVISION_ID     0x0101
#endif

// **************************************

#define USB_VENDOR_ID_HI  (USB_VENDOR_ID >> 8)
#define USB_VENDOR_ID_LO  (USB_VENDOR_ID & 0x00FF)
#define USB_PRODUCT_ID_HI (USB_PRODUCT_ID >> 8)
#define USB_PRODUCT_ID_LO (USB_PRODUCT_ID & 0x00FF)
#define USB_REVISION_ID_HI  (USB_REVISION_ID >> 8)
#define USB_REVISION_ID_LO  (USB_REVISION_ID & 0x00FF)
#define USB_SPEC_ID_HI    (USB_SPEC_ID >> 8)
#define USB_SPEC_ID_LO    (USB_SPEC_ID & 0x00FF)


extern int usb_ioctl(struct inode *inodep, struct file *filep, unsigned int cmd, unsigned long arg);
extern int usb_open(struct inode *inodp, struct file *filp);
extern int usb_close(struct inode *inodp, struct file *filp);
extern ssize_t usb_read(struct file *filp, char *userBuffer, size_t userCount, loff_t *loff);
extern ssize_t usb_write(struct file *filp, const char *userBuffer, size_t userCount, loff_t *loff);

#endif
