/* ------------------------------------------------------------------------- */
/* i2c-etx-p1.c i2c-hw access for bitbanging i2c adapters                    */
/* ------------------------------------------------------------------------- */
/*   Copyright (C) 1995-2000 Simon G. Vogl

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.		     */
/* ------------------------------------------------------------------------- */ 

/* With some changes from Ky�ti M�kki <kmalkki@cc.hut.fi> and even
   Frodo Looijaard <frodol@dds.nl> */

/* $Id: i2c-philips-par.c,v 1.18 2000/07/06 19:21:49 frodo Exp $ */

#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/stddef.h>
#include <linux/config.h>
#include <linux/pci.h>

#include <linux/i2c.h>
#include <linux/i2c-algo-bit.h>

#include "i2c-etx_p1.h"

#ifndef __exit
#define __exit __init
#endif

// PCI device issues
struct pci_dev *ppci_dev = NULL;
unsigned pci_base;  // physical pci base of device data ram

// i2c adapter structure
struct i2c_adapter adapter;

/* ----- global defines -----------------------------------------------	*/
#define DEB(x)		/* should be reasonable open, close &c. 	*/
#define DEB2(x) 	/* low level debugging - very slow 		*/
#define DEBE(x)	x	/* error messages 				*/

/* ----- local functions ----------------------------------------------	*/
static void gpio_setscl(void *dev, int state)
{
  u8 byte;

  if (state)
  {
    pci_read_config_byte(dev, GPIO_DIRECTION_REG, &byte);
    byte &= ~I2CCLK_MASK;
    pci_write_config_byte(dev, GPIO_DIRECTION_REG, byte);
  }
  else
  {
    pci_read_config_byte(dev, GPIO_DIRECTION_REG, &byte);
    byte |= I2CCLK_MASK;
    pci_write_config_byte(dev, GPIO_DIRECTION_REG, byte);

    pci_read_config_byte(dev, GPIO_DATA_WRITE_REG, &byte);
    byte &= ~I2CCLK_MASK;
    pci_write_config_byte(dev, GPIO_DATA_WRITE_REG, byte);
  }
}

static void gpio_setsda(void *dev, int state)
{
  u8 byte;

  if (state)
  {
    pci_read_config_byte(dev, GPIO_DIRECTION_REG, &byte);
    byte &= ~I2CDAT_MASK;
    pci_write_config_byte(dev, GPIO_DIRECTION_REG, byte);
  }
  else
  {
    pci_read_config_byte(dev, GPIO_DIRECTION_REG, &byte);
    byte |= I2CDAT_MASK;
    pci_write_config_byte(dev, GPIO_DIRECTION_REG, byte);

    pci_read_config_byte(dev, GPIO_DATA_WRITE_REG, &byte);
    byte &= ~I2CDAT_MASK;
    pci_write_config_byte(dev, GPIO_DATA_WRITE_REG, byte);
  }
}

static int gpio_getscl(void *dev)
{
  u8 byte;

  pci_read_config_byte(dev, GPIO_DIRECTION_REG, &byte);
  byte &= ~I2CCLK_MASK;
  pci_write_config_byte(dev, GPIO_DIRECTION_REG, byte);

  pci_read_config_byte(dev, GPIO_DATA_READ_REG, &byte);

  if(byte & I2CCLK_MASK)
    return(1);
  else
    return(0);
}

static int gpio_getsda(void *dev)
{
  u8 byte;	

  pci_read_config_byte(dev, GPIO_DIRECTION_REG, &byte);
  byte &= ~I2CDAT_MASK;
  pci_write_config_byte(dev, GPIO_DIRECTION_REG, byte);

  pci_read_config_byte(dev, GPIO_DATA_READ_REG, &byte);

  if(byte & I2CDAT_MASK)
    return(1);
  else	
    return(0);
}

static int gpio_reg(struct i2c_client *client)
{
  return 0;
}

static int gpio_unreg(struct i2c_client *client)
{
  return 0;
}

static void gpio_inc_use(struct i2c_adapter *adap)
{
  MOD_INC_USE_COUNT;
}

static void gpio_dec_use(struct i2c_adapter *adap)
{
  MOD_DEC_USE_COUNT;
}

/* ------------------------------------------------------------------------
 * Encapsulate the above functions in the correct operations structure.
 * This is only done when more than one hardware adapter is supported.
 */
 
static struct i2c_algo_bit_data gpio_data = {
  NULL,
  gpio_setsda,
  gpio_setscl,
  gpio_getsda,
  gpio_getscl,
  80, 80, 100,  /* waits, timeout */
}; 

static struct i2c_adapter gpio_ops = {
  "Jumptec ETX-P1 Aladdin V adapter",
  I2C_HW_B_LP,
  NULL,
  NULL,
  gpio_inc_use,
  gpio_dec_use,
  gpio_reg,
  gpio_unreg,
};


int pci_init_dev(struct pci_dev *dev, int *base0)
{
  int retcode;

  if ((retcode=pci_read_config_dword(dev, PCI_BASE_ADDRESS_0, base0)) == 0)
    ;//pci_set_master(dev);

  return retcode;
}


int __init i2c_bitgpio_init(void)
{
  int retcode;

  printk("i2c-gpio.o: ETX P1 Aladdin V i2c adapter module\n");

  // find PCI on pci bus and initialize it
  if (!pci_present())	
  {
    printk("no PCI bios present! device cannot be accessed ;-(\n");	
    return -EIO;	
  }
  if ((ppci_dev=pci_find_device(PCI_VENDOR, PCI_DEVICE, ppci_dev)))	
  {
    if ((retcode=pci_init_dev(ppci_dev, (u32*)&pci_base)) != 0)	
    {
      printk("Error on PCI initialisation: %x\n", retcode);	
      return -EIO;	
    }
    else	
      printk("found PCI. Vendor: %x, Device: %x\n", PCI_VENDOR, PCI_DEVICE);	
  }
  else	
  {
    printk("cannot find PCI device ;-(\n");
    return -EIO;
  }

  adapter = gpio_ops;
  adapter.algo_data = &gpio_data;
  gpio_data.data = ppci_dev;

  //gpio_setsda(ppci_dev, 0);    // to see the toggle
  //gpio_setscl(ppci_dev, 0);
  //gpio_setsda(ppci_dev, 1);
  //gpio_setscl(ppci_dev, 1);

  if (i2c_bit_add_bus(&adapter) < 0)
  {
    printk("i2c-gpio.o: Unable to register with I2C.\n");
    return -EIO;   /* No good */
  }

  return 0;
}


void __exit i2c_bitgpio_exit(void)
{
  i2c_bit_del_bus(&adapter);
}


EXPORT_NO_SYMBOLS;

MODULE_AUTHOR("Arno Geissel HBM <arno.geissel@hbm.com>");
MODULE_DESCRIPTION("I2C-Bus adapter routines for Jumptec ETX P1 Aladdin V Board");

//MODULE_PARM(type, "i");

#ifdef MODULE
int init_module(void)
{
  return i2c_bitgpio_init();
}

void cleanup_module(void)
{
  i2c_bitgpio_exit();
}
#endif
