/***************************************************************************
                                  buffer.c
                              -------------------
    project:  USB device for Philips ISP1181A

    begin                : Thu Dec 12 2002
    copyright            : (C) 2002 by HBM GmbH, Siegfried Wessler
    email                : siegfried.wessler@hbm.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include  <asm/io.h>
#include  <asm/uaccess.h> // copy_from_user()...
#include  <linux/poll.h>

#include  "types.h"
#include  "global.h"
#include  "isp1181.h"
#include  "descript.h"
#include  "usbdev.h"



//  **** functions called from main.c:

UINT16  calc_buffersizes(UINT16 setfifo, UINT8 minor_no)
{
// It is assumed for AlternateSetting==1 that we always use no double buffers for bulk and always double buffers for iso endpoints
//   and for AlternateSetting==0 we only use double buffering for the bulk IN endpoint.

UINT16  iso_fifosum, noniso_fifosum, reduced_fifosum;
INT16   fifo_left;
UINT16  newBULKsize;
UINT8   btemp;
INT16   ret = 0;
UINT8   i;

  iso_fifosum = 0;
  for (i=EP_ISO1; i<=LAST_MAIN_EP_NUMBER; i++)
  {
    if ((i -EP_ISO1 +1) == minor_no)    // e.g. endpoint 4 -> minor 1
      iso_fifosum += (2* isp_GetIsoFifoSize(setfifo, &btemp));  // two times, because iso endpoint are always double buffered
    else
      iso_fifosum += (2* isp_GetIsoFifoSize(EndpointDescrAS1[i-(EP_ISO1-2)].wMaxPacketSize, &btemp));   // e.g. endpoint 4 -> array index 2
  }

  if (EnableBulkDoubleBuffer)
    noniso_fifosum = EP_CTRL_OUT_FIFO_SIZE +EP_CTRL_IN_FIFO_SIZE + 2*EP_MAIN_IN_DEFAULT_FIFO_SIZE +EP_MAIN_OUT_DEFAULT_FIFO_SIZE;
  else
    noniso_fifosum = EP_CTRL_OUT_FIFO_SIZE +EP_CTRL_IN_FIFO_SIZE + EP_MAIN_IN_DEFAULT_FIFO_SIZE +EP_MAIN_OUT_DEFAULT_FIFO_SIZE;

  fifo_left = MAX_FIFOSUM -noniso_fifosum -iso_fifosum;

  if (fifo_left >= 0)
  { // all definitions of wMaxPacketSize perfectly fit into hardware fifos of ISP1181:
    EndpointDescrAS1[EP_MAIN_IN -2].wMaxPacketSize = EP_MAIN_IN_DEFAULT_FIFO_SIZE;
    EndpointDescrAS1[EP_MAIN_OUT -2].wMaxPacketSize = EP_MAIN_OUT_DEFAULT_FIFO_SIZE;
    EndpointDescrAS1[minor_no+1].wMaxPacketSize = setfifo;
    calc_endpoints();
    ret = 0;
    reduced_fifosum = 0;
  }
  else
  { // wMaxPacketSize is greater than hardware fifo can offer, try reducing BULK fifo size:
    if (fifo_left >= -64)
      // reduce BULK fifo to 2*32 bytes:
      newBULKsize = 32;
    else
      if (fifo_left >= -96)
        // reduce BULK fifo to 2*16 bytes:
        newBULKsize = 16;
      else
        if (fifo_left >= -112)
          // reduce BULK fifo to 2*8 bytes:
          newBULKsize = 8;
        else
          newBULKsize = 0;

    if (newBULKsize)
    {
      EndpointDescrAS1[EP_MAIN_IN -2].wMaxPacketSize = newBULKsize;
      EndpointDescrAS1[EP_MAIN_OUT -2].wMaxPacketSize = newBULKsize;
      EndpointDescrAS1[minor_no+1].wMaxPacketSize = setfifo;
      calc_endpoints();
      ret = 0;
    }
    else
    { // no way to make this request fit into hardware fifos, don't change anything and reject:
      printk("%s: Fifos could not be configured. Fifosum: requested=%d, missing=%d\n", MODULE_NAME, setfifo, -fifo_left);
      ret = -1;
    }
  }

  printk("%s: dynamically configured endpoints for AlternateSetting 1 are:\n", MODULE_NAME);
  for (i = FIRST_MAIN_EP_NUMBER; i < MAX_ENDPOINTS; i++)
  {
    if (EndpointDescrAS1[i -2].wMaxPacketSize)
    {
      printk("  EP 0x%02X: ", EndpointDescrAS1[i -2].bEndpointAddress);
      switch (EndpointDescrAS1[i -2].bmAttributes)
      {
        case USB_ATTR_BULK:     printk("BULK ");
                                if (EndpointDescrAS1[i -2].bEndpointAddress & USB_ADDR_DIR_IN)
                                  printk("IN ");
                                else
                                  printk("OUT");
                                printk("  wMaxBufferSize=%-4d  ISP1181A fifo used=%d\n", EndpointDescrAS1[i -2].wMaxPacketSize, (isp_GetNonIsoFifoSize(EndpointDescrAS1[i -2].wMaxPacketSize, &btemp)));
                                break;

        case USB_ATTR_ISOCHRON: printk("ISO ");
                                if (EndpointDescrAS1[i -2].bEndpointAddress & USB_ADDR_DIR_IN)
                                  printk("IN ");
                                else
                                  printk("OUT");
                                printk("   wMaxBufferSize=%-4d  ISP1181A fifo used=%d\n", EndpointDescrAS1[i -2].wMaxPacketSize, (2* isp_GetIsoFifoSize(EndpointDescrAS1[i -2].wMaxPacketSize, &btemp)));
                                break;
      }
    }

  }

  return ret;

}   // calc_buffersizes


void  InitMAINBuffer(void)
{
  MAINBuffer.pWriteAppl = MAINBuffer.INBuffer;
  MAINBuffer.pSendUSB = MAINBuffer.INBuffer;
  MAINBuffer.pReadAppl = MAINBuffer.OUTBuffer;
  MAINBuffer.pGetUSB = MAINBuffer.OUTBuffer;
  MAINBuffer.EndpointNoIN = EP_MAIN_IN;
  MAINBuffer.EndpointNoOUT = EP_MAIN_OUT;
  MAINBuffer.FullblockWritten = FALSE;
  MAINBuffer.FifoSizeIN = BULK_MAX_PACKET_SIZE;
  MAINBuffer.FifoSizeOUT = BULK_MAX_PACKET_SIZE;

  init_waitqueue_head(&MAINBuffer.waitqueue);

}  // InitMainBuffer



INT32 ApplWriteIsoBuffer(tDevice *dev, const char *userBuffer, INT32 userCount)
{
UINT8   i;
INT16   CopyCount;
tIsoBuffer  *WrBuffer;
INT32   return_value;

  if (userCount > IsoBufMax)
    CopyCount = IsoBufMax;
  else
    CopyCount = userCount;

  i = dev->minor_no -1;

  WrBuffer = WrIsoBuffer[i];

  if (WrBuffer->Status == 0)
  { // this buffer is empty, use it:

    if (copy_from_user(WrBuffer->Buffer, userBuffer, CopyCount) != 0)
    { // error while copying:
      return_value = -1;
    }
    else  // copying successful:
    {
      return_value = CopyCount;
      WrBuffer->ToSend = CopyCount;
      WrBuffer->Status = 1;

      if (WrBuffer->Index == 1)
      {
        WrIsoBuffer[i] = &IsoBufferB[i];
      }
      else
      {
        WrIsoBuffer[i] = &IsoBufferA[i];
      }

      // To prevent race errors, all access to ISP except initialization is done within one tasklet:
      tasklet_jobs |= (ISP_INT_ISO1 << i);  // in tasklet fill isp fifo if possible
    }

  }
  else
  {
    printk("%s: warning: application write call to /usbdev%d: internal buffers full.\n", MODULE_NAME, i+1);
    return_value = 0;
  }

  return return_value;

}   // ApplWriteIsoBuffer




INT32 ApplWriteBulkBuffer(tBulkBuffer *BulkData, const char *userBuffer, INT32 userCount)
{
INT32   toCopy;
INT32   bytes_written;
INT32   free_right, free_left;
UINT8   *pWrite, *pSend;
UINT8   *pINBuffer, *pOUTBuffer;


  printd("ApplWriteBulk (%ld):%.20s\n", userCount, userBuffer);

  pWrite = BulkData->pWriteAppl;
  pSend = BulkData->pSendUSB;

  pINBuffer = BulkData->INBuffer;
  pOUTBuffer = BulkData->OUTBuffer;

  if (pWrite >= pINBuffer +INBufMax)
  {
    if (pSend > pINBuffer +1)
    { // pWriteAppl is at the right boundary and pSendUSB is not at the very first two bytes
      pWrite = pINBuffer;
    }
    else
      return 0;
  }

  if (pSend <= pWrite)
  { // pWriteAppl is right of pSendUSB. if pSendUSB==pWriteAppl all data were already transmitted:
    free_right = (pINBuffer -pWrite) +INBufMax;

    if (userCount <= free_right)
    {
      if (copy_from_user(pWrite, userBuffer, userCount) != 0)
        return 0;
      else
      {
        pWrite += userCount;
        bytes_written = userCount;
      }
    }
    else
    { // userBuffer doesn't fit into right part of INBuffer.
      // get first part of userBuffer:
      if (copy_from_user(pWrite, userBuffer, free_right) != 0)
        return 0;
      else
      { // now see how much of the second part of userBuffer can be read:
        bytes_written = free_right;

        if (pSend <= pINBuffer +1)
        { // if pSend is at the very left boundary, there is no space available
          pWrite += free_right;
        }
        else
        {
          pWrite = pINBuffer;   // set pWrite to left boundary
          free_left = (pSend -pINBuffer) -1;  // always keep one byte difference!
          toCopy = userCount -free_right;

          if (free_left -toCopy >= 0)
          { // if at left boundary is enough space, get all:
            if (copy_from_user(pWrite, userBuffer +free_right, toCopy) != 0)
              return 0;
            else
            {
              pWrite += toCopy;
              bytes_written += toCopy;
            }
          }
          else
          { // pSend is too close, don't get them all:
            if (copy_from_user(pWrite, userBuffer +free_right, free_left) != 0)
              return 0;
            else
            {
              pWrite += free_left;
              bytes_written += free_left;
            }
          }
        }
      }
    }
  }
  else
  { // when usb_write was called pWriteAppl was left of pSendUSB:
    free_right = (pSend -pWrite) -1;  // always keep one byte difference!

    if (free_right == 0)
      return 0;
    else
    {
      if (free_right > userCount)
        free_right = userCount;

      if (copy_from_user(pWrite, userBuffer, free_right) != 0)
        return 0;
      else
      {
        pWrite += free_right;
        bytes_written = free_right;
      }
    }
  }

  BulkData->pWriteAppl = pWrite;

  if (BulkData->EndpointNoIN == EP_MAIN_IN)
    // To prevent race errors, all access to ISP except initialization is done within one tasklet:
    tasklet_jobs |= ISP_INT_MAIN_IN;  // directly feed usb chip iso fifo buffers, if possible

  return  bytes_written;

} //  ApplWriteBulkBuffer



INT32 ApplReadBulkBuffer(tBulkBuffer *BulkData, char *userBuffer, INT32 userCount)
{
INT32   bytes_read;
UINT8   *pRead, *pGet;
UINT8   *pBuffer;
INT32   toCopy;
UINT8   *pINBuffer, *pOUTBuffer;

  pRead = BulkData->pReadAppl;
  pGet = BulkData->pGetUSB;

  pINBuffer = BulkData->INBuffer;
  pOUTBuffer = BulkData->OUTBuffer;

  if (pRead == pGet)
  { // no data to return to application
    return 0;
  }

  if (pRead >= pOUTBuffer +OUTBufMax)
  { // pReadAppl at the very right boundary:
    if (pGet > pOUTBuffer)
    {
      pRead = pOUTBuffer;
    }
    else
      return 0;
  }

  pBuffer = &BulkData->TempReadBuffer[0];

  if (pRead < pGet)
  {
    toCopy = pGet -pRead;

    if (toCopy > userCount)
      toCopy = userCount;

    if (copy_to_user(userBuffer, pRead, toCopy) != 0)
      return 0;
    else
      pRead += toCopy;

    bytes_read = toCopy;
  }
  else
  { // pRead > pGet
    toCopy = OUTBufMax -(pRead -pOUTBuffer);

    if (toCopy > userCount)
    { // user doesn't want that much data; don't do a wrap:
      toCopy = userCount;
      if (copy_to_user(userBuffer, pRead, toCopy) != 0)
        return 0;
      else
        pRead += toCopy;
      bytes_read = toCopy;
    }
    else
    { // okay, user can handle this so far, so get everything right of pRead:
      pBuffer = &BulkData->TempReadBuffer[0];
      memcpy(pBuffer, pRead, toCopy);

      pRead += toCopy;
      pBuffer += toCopy;
      bytes_read = toCopy;

      if (pGet > pOUTBuffer)
      { // there is some data at the very left of OUTBuffer valid
        toCopy = pGet -pOUTBuffer;

        if (bytes_read +toCopy > userCount)
        { // user can't handle everything, only add as much as possible to the buffer:
          toCopy = userCount -bytes_read;
        }

        memcpy(pBuffer, pOUTBuffer, toCopy);
        pRead = pOUTBuffer +toCopy;
        bytes_read += toCopy;
      }

      if (copy_to_user(userBuffer, &BulkData->TempReadBuffer[0], bytes_read) != 0)
      {
        return 0;
      }
    }

  }

  BulkData->pReadAppl = pRead;

  return bytes_read;

} //  ApplReadBulkBuffer






//  **** functions called from isr.c:

spinlock_t Lock_FeedBulkIN;   // this is global, so FeedBulkINFifo is always called only once.


void  FeedBulkINFifo(tBulkBuffer *BulkData)
{
INT32   ret_transmitted;
INT32   filled;
UINT8   *pWrite, *pSend;
UINT8   *pBuffer;
INT32   toCopy;
UINT8   *pINBuffer;
UINT8   status;


  spin_lock(&Lock_FeedBulkIN);

  // because of spinlock we always have to get an actual status here:
  status = usb_wrc_rd1b(ISP_EP_CHECK +BulkData->EndpointNoIN);  // not clearing interrupt bit

printd1("FeedBulkIN: status=%02X\n", status);

  if ( ( !(EnableBulkDoubleBuffer)
       && ( (status & (ISP_EP_STATUS_EPFULL1 | ISP_EP_STATUS_EPFULL0)) == 0 )
       )  // double buffering disabled: both fifos have to be empty
     ||( (EnableBulkDoubleBuffer)
       && (status & (ISP_EP_STATUS_EPFULL1 | ISP_EP_STATUS_EPFULL0))
            != (ISP_EP_STATUS_EPFULL1 | ISP_EP_STATUS_EPFULL0)
       )  // double buffering enabled: only one fifo has to be empty
     )
  {

    pWrite = BulkData->pWriteAppl;
    pSend = BulkData->pSendUSB;

    pINBuffer = &BulkData->INBuffer[0];

    if (pWrite == pSend)
      isp_WriteBulkEndpoint(BulkData, pINBuffer, 0);
    else
    {

      if (pSend >= pINBuffer +INBufMax)
      {
        if (pWrite > pINBuffer)
          pSend = pINBuffer;
        else
        {
          isp_WriteBulkEndpoint(BulkData, pINBuffer, 0);
          return;
        }
      }

      filled = pWrite -pSend;

      if (filled < 0)
      { // pSend is right of pWrite:
        filled = (pINBuffer -pSend) +INBufMax;

        if (filled >= BulkData->FifoSizeIN)
        {
          ret_transmitted = isp_WriteBulkEndpoint(BulkData, pSend, BulkData->FifoSizeIN);
          pSend += ret_transmitted;
        }
        else
        { // ISP fifo buffer is bigger than data right of pSend, get all data right of pSend...:
          pBuffer = &BulkData->TempINBuffer[0];
          memcpy(pBuffer, pSend, filled);
          toCopy = filled;
          pSend += filled;

          // ...and then try to get some from the left boundary of INBuffer:
          pBuffer += filled;
          filled = pWrite -pINBuffer;
          if (toCopy +filled > BulkData->FifoSizeIN)
            filled = BulkData->FifoSizeIN -toCopy;
          memcpy(pBuffer, pINBuffer, filled);
          toCopy += filled;
          pSend = pINBuffer +filled;

          // finally try to transmit this buffer:
          ret_transmitted = isp_WriteBulkEndpoint(BulkData, &BulkData->TempINBuffer[0], toCopy);
        }
      }
      else
      {
        ret_transmitted = isp_WriteBulkEndpoint(BulkData, pSend, filled);
        pSend += ret_transmitted;
      }

    BulkData->pSendUSB = pSend;

    }
  }
  else
  {
    printd(" ISP fifo not empty!\n"); ;
  }

  spin_unlock(&Lock_FeedBulkIN);

  return;

} //  FeedBulkINBuffer



void GetBulkOUTFifo(tBulkBuffer *BulkData, UINT16 status)
{
INT32  toCopy;
UINT8  *pBuffer;
UINT8  *pRead;
UINT8  *pGet;
INT32  free_right;
UINT8  *pINBuffer, *pOUTBuffer;

  pRead = BulkData->pReadAppl;
  pGet = BulkData->pGetUSB;
  pBuffer = &BulkData->TempOUTBuffer[0];
  pOUTBuffer = &BulkData->OUTBuffer[0];
  pINBuffer = &BulkData->INBuffer[0];

  toCopy = isp_GetBulkEndpoint(BulkData, pBuffer, BulkData->FifoSizeOUT);

  printd1("GetBulkOUTFifo (%ld):%.4s\n", toCopy, pBuffer);

  if (toCopy > 0)
  { // some data received from USB, now copy them into ring buffer:

    if (pGet >= pOUTBuffer +OUTBufMax)
    { // pGetUSB at very right boundary:
      if (pRead > pOUTBuffer +1)
      { // there is still some space left of pReadAppl, so set pGetUSB at the left boundary:
        pGet = pOUTBuffer;
      }
      else
      {
        printk("%s:  BulkOUTBuffer full. Application should read this device!\n", MODULE_NAME);
        wake_up_interruptible(&MAINBuffer.waitqueue); // it's time that user read call gets some data!
        return; // OUTBuffer is totally filled. All USB OUT packets written to Buffer are lost.
      }
    }

    if (pGet >= pRead)
    { // PGetUSB is right of pReadAppl
      free_right = OUTBufMax -(pGet -pOUTBuffer);

      if (free_right >= toCopy)
      { // there's enough space right of pGetUSB, directly copy everything:

        memcpy(pGet, pBuffer, toCopy);
        pGet += toCopy;
      }
      else
      {
        memcpy(pGet, pBuffer, free_right);
        pGet += free_right;
        pBuffer += free_right;
        toCopy -= free_right;

        if (pRead > pOUTBuffer +1)
        { // there is some space left of pReadAppl:
          pGet = pOUTBuffer; // place pGet at very left boundary
          free_right = pRead -pOUTBuffer -1;

          if (free_right < toCopy)
            toCopy = free_right;

          memcpy(pGet, pBuffer, toCopy);
          pGet += toCopy;
        }
        else
        {
          printk("%s:  BulkOUTBuffer full. Application should read this device!\n", MODULE_NAME);
          wake_up_interruptible(&MAINBuffer.waitqueue); // it's time that user read call gets some data!
          return;   // discard the whole last USB transmission;
        }
      }
    }
    else
    { // pGet < pRead
      free_right = (pRead -pGet) -1;

      if (free_right < toCopy)
      {
        wake_up_interruptible(&MAINBuffer.waitqueue); // it's time that user read call gets some data!
        return;   // discard the whole last USB transmission;
      }

      memcpy(pGet, pBuffer, toCopy);
      pGet += toCopy;
    }

  BulkData->pGetUSB = pGet;

  wake_up_interruptible(&MAINBuffer.waitqueue);   // let the user read call get some data

  }

} //  GetBulkOUTFifo



spinlock_t Lock_FeedIso;   // this is global, so FeedIsoFifo is always called only once.


void FeedIsoFifo(UINT8 i)
{
tIsoBuffer *RdBuffer;
UINT8   status;
INT32   sent;

  spin_lock(&Lock_FeedIso);

  RdBuffer = RdIsoBuffer[i];

  if (RdBuffer->Status)
  { // there is something to transmit:

    // because of spin locking we always have to get an actual status here:
    status = usb_wrc_rd1b(ISP_EP_CHECK +EP_ISO1 +i);

    printd("B.status=%d, B.index=%d, B.ToSend=%ld, isp_status=0x%04X\n", RdBuffer->Status, RdBuffer->Index, RdBuffer->ToSend, status);

    if ((status & (ISP_EP_STATUS_EPFULL1 | ISP_EP_STATUS_EPFULL0)) != (ISP_EP_STATUS_EPFULL1 | ISP_EP_STATUS_EPFULL0))
    {
      sent = isp_WriteIsoEndpoint(EP_ISO1 +i, RdBuffer->Buffer +RdBuffer->Offset, RdBuffer->ToSend);
      RdBuffer->ToSend -= sent;
      RdBuffer->Offset += sent;
      if (RdBuffer->ToSend < 1)
      { // all data sent
        RdBuffer->ToSend = RdBuffer->Status = RdBuffer->Offset = 0;

        if (RdBuffer->Index == 1)
          RdIsoBuffer[i] = &IsoBufferB[i];
        else
          RdIsoBuffer[i] = &IsoBufferA[i];
      }
    }
  }

  spin_unlock(&Lock_FeedIso);

}   // FeedIsoFifo


