/***************************************************************************
                                  isp1181.h
                              -------------------
    project:  USB device for Philips ISP1181A

    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by HBM GmbH, Siegfried Wessler
    email                : siegfried.wessler@hbm.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef __ISP1181__
#define __ISP1181__

//#include "../../cpxxapp/cp64/target.h"
#define   CP42

#include  <asm/irq.h>




// Defines depending on hardware layout:
#ifdef CP42
  #define USB_IRQ_NO      6                     // IRQ no for USB chip
  #define DRIVER_TARGET   "CP42"
  #define portbase_addr   0x278                 // base address to port of USB chip
  #define CHIPADDR        2                     // Address line no. for USB Chip
  #define DMA_CHANNEL     5
#endif

#ifdef CP22
  #define USB_IRQ_NO      15                    // IRQ no for USB chip
  #define DRIVER_TARGET   "CP22"

  #define portbase_addr   0x278                 // base address to port of USB chip
  #define CHIPADDR        2                     // Address line no. for USB Chip
  #define DMA_CHANNEL     1

#endif




// #######################################

#define DISABLE_INTERRUPT   disable_irq(USB_IRQ_NO)
#define ENABLE_INTERRUPT    enable_irq(USB_IRQ_NO)


// Set USB_IRQ_NO to level sensitive, since it is not by default, I found out reading 0x4D1!
// See ZFx86 data book 1.0 Rev B, Table 4.45, works just the same on ETX board for CP42.
// Check Table 4.52 for interrupt polarity too!
#define SET_INTERRUPT_LEVEL outb( (inb((USB_IRQ_NO > 7) ? 0x4D1 : 0x4D0) | (0x01 << (USB_IRQ_NO %8))), (USB_IRQ_NO > 7) ? 0x4D1 : 0x4D0)

#define portbase_len    (2* (1 << CHIPADDR))   // bytes of port needed

#define ISP_COMMAND     (portbase_addr +(1 << CHIPADDR)) // command address to USB chip
#define ISP_DATA        (portbase_addr +0)               // data address to USB chip



// Defines depending on USB chip:
#define CLOCKDIV        7

#define EP_CTRL_OUT_FIFO_SIZE   64
#define EP_CTRL_IN_FIFO_SIZE    64
#define EP_CTRL_PACKET_SIZE     64

#define EP_MAIN_OUT_DEFAULT_FIFO_SIZE   64
#define EP_MAIN_IN_DEFAULT_FIFO_SIZE    64
#define BULK_MAX_PACKET_SIZE    64    // fifo size for BULK arranged dynamically for AlternateSetting 1

#define MAX_FIFOSUM             2462

// ISP1181 command addresses and values:

#define EP_CTRL_OUT                   0
#define EP_CTRL_IN                    1
#define EP_MAIN_IN                    2
#define EP_MAIN_OUT                   3
#define EPX3                          4
#define EPX4                          5
#define EPX5                          6
#define EPX6                          7
#define EPX7                          8
#define EPX8                          9
#define EPX9                          10
#define EPX10                         11
#define EPX11                         12
#define EPX12                         13
#define EPX13                         14
#define EPX14                         15

#define EP_ISO1                       EPX3
#define EP_ISO2                       EPX4
#define EP_ISO3                       EPX5


#define ISP_WR_ECR                    0x20    // endpoint configuration registers
#define ISP_WR_ECR_CTRL_OUT             ISP_WR_ECR +EP_CTRL_OUT   // receive from usb host
#define ISP_WR_ECR_CTRL_IN              ISP_WR_ECR +EP_CTRL_IN    // transmit to usb host
#define ISP_WR_ECR_MAIN_IN              ISP_WR_ECR +EP_MAIN_IN
#define ISP_WR_ECR_MAIN_OUT             ISP_WR_ECR +EP_MAIN_OUT
#define ISP_RD_ECR                    0x30
#define ISP_RD_ECR_CTRL_OUT             ISP_RD_ECR +EP_CTRL_OUT
#define ISP_RD_ECR_CTRL_IN              ISP_RD_ECR +EP_CTRL_IN
#define ISP_RD_ECR_MAIN_IN              ISP_RD_ECR +EP_MAIN_IN
#define ISP_RD_ECR_MAIN_OUT             ISP_RD_ECR +EP_MAIN_OUT
#define   ISP_ECR_FIFOEN                  0x80    // 1: fifo enabled, 0: fifo disabled
#define   ISP_ECR_EPDIR_IN                0x40    // 1: direction IN, 0: direction OUT
#define   ISP_ECR_DBLBUF                  0x20    // 1: double buffering enabled
#define   ISP_ECR_FFOISO                  0x10    // 1: isochronous, 0: bulk or interrupt
#define   ISP_ECR_FFOSZ_NI8                 0x00
#define   ISP_ECR_FFOSZ_NI16                0x01
#define   ISP_ECR_FFOSZ_NI32                0x02
#define   ISP_ECR_FFOSZ_NI64                0x03
#define   ISP_ECR_FFOSZ_I16                 0x00
#define   ISP_ECR_FFOSZ_I32                 0x01
#define   ISP_ECR_FFOSZ_I48                 0x02
#define   ISP_ECR_FFOSZ_I64                 0x03
#define   ISP_ECR_FFOSZ_I96                 0x04
#define   ISP_ECR_FFOSZ_I128                0x05
#define   ISP_ECR_FFOSZ_I160                0x06
#define   ISP_ECR_FFOSZ_I192                0x07
#define   ISP_ECR_FFOSZ_I256                0x08
#define   ISP_ECR_FFOSZ_I320                0x09
#define   ISP_ECR_FFOSZ_I384                0x0A
#define   ISP_ECR_FFOSZ_I512                0x0B
#define   ISP_ECR_FFOSZ_I640                0x0C
#define   ISP_ECR_FFOSZ_I768                0x0D
#define   ISP_ECR_FFOSZ_I896                0x0E
#define   ISP_ECR_FFOSZ_I1023               0x0F

#define ISP_WR_ADDR                   0xB6
#define ISP_RD_ADDR                   0xB7
#define   ISP_ADDR_DEVEN                  0x80

#define ISP_WR_MODE                   0xB8    // mode register
#define ISP_RD_MODE                   0xB9
#define   ISP_MODE_DMAWD                  0x80
#define   ISP_MODE_GOSUSP                 0x20
#define   ISP_MODE_INTENA                 0x08
#define   ISP_MODE_DBGMOD                 0x04
#define   ISP_MODE_SOFTCT                 0x01

#define ISP_WR_IER                    0xC2    // interrupt enable register
#define ISP_RD_IER                    0xC3
#define ISP_RD_INT_REG                0xC0    // read interrupt register
#define   ISP_INT_BASE_BIT                0x00000100
#define   ISP_INT_CTRL_OUT                (ISP_INT_BASE_BIT << EP_CTRL_OUT)
#define   ISP_INT_CTRL_IN                 (ISP_INT_BASE_BIT << EP_CTRL_IN)
#define   ISP_INT_MAIN_IN                 (ISP_INT_BASE_BIT << EP_MAIN_IN)
#define   ISP_INT_MAIN_OUT                (ISP_INT_BASE_BIT << EP_MAIN_OUT)
#define   ISP_INT_ISO1                    (ISP_INT_BASE_BIT << EP_ISO1)
#define   ISP_INT_ISO2                    (ISP_INT_BASE_BIT << EP_ISO2)
#define   ISP_INT_ISO3                    (ISP_INT_BASE_BIT << EP_ISO3)
#define   ISP_INT_BUSTATUS                0x00000080
#define   ISP_INT_PSOF                    0x00000020
#define   ISP_INT_SOF                     0x00000010
#define   ISP_INT_EOT                     0x00000008
#define   ISP_INT_SUSPND                  0x00000004
#define   ISP_INT_RESUME                  0x00000002
#define   ISP_INT_RESET                   0x00000001

#define ISP_WR_HCR                    0xBA    // hardware configuration register
#define ISP_RD_HCR                    0xBB
#define   ISP_HCR_EXTPUL                  0x4000
#define   ISP_HCR_NOLAZY                  0x2000
#define   ISP_HCR_CLKRUN                  0x1000
#define   ISP_HCR_CKDIV                   (CLOCKDIV << 8)
#define   ISP_HCR_DAKOLY                  0x0080
#define   ISP_HCR_DRQPOL                  0x0040
#define   ISP_HCR_DAKPOL                  0x0020
#define   ISP_HCR_EOTPOL                  0x0010
#define   ISP_HCR_WKUPCS                  0x0008
#define   ISP_HCR_PWROFF                  0x0004
#define   ISP_HCR_INTLVL                  0x0002
#define   ISP_HCR_INTPOL                  0x0001

#define ISP_WR_DMACONFIG              0xF0    // dma configuartion register
#define ISP_RD_DMACONFIG              0xF1

#define   ISP_DMA_CNTREN                  0x8000
#define   ISP_DMA_SHORTP                  0x4000
#define   ISP_DMA_DMAEN                   0x0008
#define   ISP_DMA_BURST1                0x0000
#define   ISP_DMA_BURST4                0x0001
#define   ISP_DMA_BURST8                0x0002
#define   ISP_DMA_BURST16               0x0003

#define ISP_WR_DMACOUNT               0xF2    // dma counter register
#define ISP_RD_DMACOUNT               0xF3

#define ISP_RESET                     0xF6    // device reset

#define ISP_WR_EPBUF                  0x00    // write endpoint buffer
#define ISP_RD_EPBUF                  0x10    // read endpoint buffer
#define ISP_RD_EPBUF_CTRL_OUT           ISP_RD_EPBUF +EP_CTRL_OUT    // receive from usb host
#define ISP_WR_EPBUF_CTRL_IN            ISP_WR_EPBUF +EP_CTRL_IN     // transmit to usb host
#define ISP_WR_EPBUF_MAIN_IN            ISP_WR_EPBUF +EP_MAIN_IN
#define ISP_RD_EPBUF_MAIN_OUT           ISP_RD_EPBUF +EP_MAIN_OUT

#define ISP_RD_EPSR                   0x50    // Read Endpoint Status Register with interrupt clear
#define ISP_RD_EPSR_CTRL_OUT            ISP_RD_EPSR +EP_CTRL_OUT
#define ISP_RD_EPSR_CTRL_IN             ISP_RD_EPSR +EP_CTRL_IN
#define ISP_RD_EPSR_MAIN_IN             ISP_RD_EPSR +EP_MAIN_IN
#define ISP_RD_EPSR_MAIN_OUT            ISP_RD_EPSR +EP_MAIN_OUT
#define ISP_RD_EPSR_ISO1                ISP_RD_EPSR +EP_ISO1
#define ISP_RD_EPSR_ISO2                ISP_RD_EPSR +EP_ISO2
#define ISP_RD_EPSR_ISO3                ISP_RD_EPSR +EP_ISO3
#define ISP_EP_CHECK                  0xD0    // check endpoint status without clearing interrupt flags
#define ISP_EP_CHECK_CTRL_OUT           ISP_EP_CHECK +EP_CTRL_OUT
#define ISP_EP_CHECK_CTRL_IN            ISP_EP_CHECK +EP_CTRL_IN
#define ISP_EP_CHECK_MAIN_IN            ISP_EP_CHECK +EP_MAIN_IN
#define ISP_EP_CHECK_MAIN_OUT           ISP_EP_CHECK +EP_MAIN_OUT
#define ISP_EP_CHECK_ISO1               ISP_EP_CHECK +EP_ISO1
#define ISP_EP_CHECK_ISO2               ISP_EP_CHECK +EP_ISO2
#define ISP_EP_CHECK_ISO3               ISP_EP_CHECK +EP_ISO3
#define   ISP_EP_STATUS_EPSTAL            0x80
#define   ISP_EP_STATUS_EPFULL1           0x40
#define   ISP_EP_STATUS_EPFULL0           0x20
#define   ISP_EP_STATUS_DATA_PID          0x10
#define   ISP_EP_STATUS_OVERWRITE         0x08
#define   ISP_EP_STATUS_SETUPT            0x04
#define   ISP_EP_STATUS_CPUBUF            0x02


#define ISP_STALL_EP                  0x40
#define ISP_STALL_CTRL_OUT              ISP_STALL_EP +EP_CTRL_OUT
#define ISP_STALL_CTRL_IN               ISP_STALL_EP +EP_CTRL_IN
#define ISP_STALL_MAIN_IN               ISP_STALL_EP +EP_MAIN_IN
#define ISP_STALL_MAIN_OUT              ISP_STALL_EP +EP_MAIN_OUT

#define ISP_UNSTALL_EP                0x80
#define ISP_UNSTALL_CTRL_OUT            ISP_UNSTALL_EP +EP_CTRL_OUT
#define ISP_UNSTALL_CTRL_IN             ISP_UNSTALL_EP +EP_CTRL_IN
#define ISP_UNSTALL_MAIN_IN             ISP_UNSTALL_EP +EP_MAIN_IN
#define ISP_UNSTALL_MAIN_OUT            ISP_UNSTALL_EP +EP_MAIN_OUT

#define ISP_VALIDATE_BUFFER           0x60    // tell usb chip to transmit to usb host
#define ISP_VALIDATE_BUFFER_CTRL_IN     ISP_VALIDATE_BUFFER +EP_CTRL_IN
#define ISP_VALIDATE_BUFFER_MAIN_IN     ISP_VALIDATE_BUFFER +EP_MAIN_IN

#define ISP_CLEAR_BUFFER              0x70    // clears endpoint receiving (from usb host) buffer
#define ISP_CLEAR_BUFFER_CTRL_OUT       ISP_CLEAR_BUFFER +EP_CTRL_OUT
#define ISP_CLEAR_BUFFER_MAIN_OUT       ISP_CLEAR_BUFFER +EP_MAIN_OUT

#define ISP_ACK_SETUP                 0xF4    // acknowledge setup

#define ISP_RD_ERROR                  0xA0    // read endpoint error register
#define ISP_RD_ERROR_CTRL_OUT           ISP_RD_ERROR +EP_CTRL_OUT
#define ISP_RD_ERROR_CTRL_IN            ISP_RD_ERROR +EP_CTRL_IN
#define ISP_RD_ERROR_MAIN_IN            ISP_RD_ERROR +EP_MAIN_IN
#define ISP_RD_ERROR_MAIN_OUT           ISP_RD_ERROR +EP_MAIN_OUT
#define   ISP_ERROR_UNREAD                0x80
#define   ISP_ERROR_DATA01                0x40
#define   ISP_ERROR_MASK                0x1E
#define   ISP_ERROR_RTOK                  0x01

#define ISP_UNLOCK                    0xB0    // unlock registers
#define   ISP_UNLOCK_DATA               0xAA37

#define ISP_WR_SCRATCH_REG            0xB2
#define ISP_RD_SCRATCH_REG            0xB3

#define ISP_READ_FRAME_NUMBER         0xB4

#define ISP_RD_CHIP_ID                0xB5    // get chip id
#define   ISP_CHIPID                    0x8141  // Chip id ISP1181A
#define   ISP_CHIPID_B                  0x8142  // Chip ID ISP1181B

//********************************

// DMA registers in CPU chipset:

#define DMA1REG_CHANNEL(c)  (2*(c))    // address of register for channel c
#define DMA1REG_COMMAND     (0x008)    // address of command register
#define DMA1REG_SMASK       (0x00A)    // address of single mask register
#define DMA1REG_MODE        (0x00B)    // address of mode register
#define DMA1REG_FFC         (0x00C)    // adress clear first/last flip flop
#define DMA1REG_MASTER_CLR  (0x00D)    // master clear


// Bits and values in DMA registers:
#define DMA37MD_SINGLE      0x40
#define DMA37MD_BLOCK       0x80
#define DMA37MD_AUTO        0x10
#define DMA37MD_DEMAND      0x00
#define DMA37MD_WRITE       0x04      // write USB chip, read memory operation
#define DMA37MD_READ        0x08      // read USB chip, write memory operation


//********************************

#define USB_IDLE            0
#define USB_TRANSMIT        1
#define USB_RECEIVE         2

// Bits in Address:
#define USB_ENDPOINT_DIRECTION_MASK   0x80    // Endpoint direction bit
#define USB_REQUEST_DIRECTION_MASK    0x80    // Request direction bit

#define USB_ENDPOINTS_MASK     (UINT8)0x0F


#endif
