/***************************************************************************
                          init.c  -  description
                             -------------------
    begin                : Thu Sep 11 2003
    copyright            : (C) 2003 by Arno Geissel
    email                : geissel@t-c-linux
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/module.h>

 #if CONFIG_MODVERSIONS==1
 #define MODVERSIONS
 #include <linux/modversions.h>
 #endif

#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <pcmcia/cs_types.h>
#include <pcmcia/cs.h>
#include <pcmcia/cistpl.h>
#include <pcmcia/cisreg.h>
#include <pcmcia/ds.h>

#include "global.h"
#include "tnt4882_regs.h"
#include "init.h"
#include "isr.h"


int  init_gpib_board(dev_link_t *link)
{
  int        i;
  int       *iptr;
  unsigned char signature;
  int32_t hw_count;

  DEBUG("init_gpib_board\n");

  /* remember this link */
  Board.link = link;

	// get irq
	if( request_irq( link->irq.AssignedIRQ, gpib_interrupt, SA_INTERRUPT, MODULE_NAME, &Board))
	{
		printk("gpib: can't request IRQ %d\n", link->irq.AssignedIRQ);
		return -1;
	}

  /* setup register map */
  if(!Board.regs)
    return -1;
  if(!Board.mirr)
  return -1;

  Board.base = link->io.BasePort1;
  iptr = &Board.regs->DIR;
  for(i=0; i<link->io.NumPorts1 && i<sizeof(sio_regs)/sizeof(int); i++)
  {
    *iptr++ = link->io.BasePort1 + i;
  }

  signature = board_in(Board.regs->CSR);
  DEBUG("chip signature: %x\n", signature);

	if ( signature & 0x83 )
	{
		printk( KERN_ERR "%s: Failed to initialize chip, not a TNT4882C or board is missing.\n", link->dev->dev_name );
		return FAIL;
	}

	/* Do software reset, release system control and then do a chip reset */	
	board_out(Board.regs->CMDR, SOFT_RESET );	// Soft Reset
	udelay(1);	


	Board.mirr->cfg = 0;                      // affected by soft reset
  Board.mirr->hssel = 0;	
		
	/* switch to 7210 mode (obsolete for this card because of SWAP) */	
  board_out(Board.regs->AUXCR, 0x80);        // compare to TNT4882 Programmer Reference, Page 4-1
  udelay(1);
  board_out(Board.regs->AUXMR, 0x80);
  udelay(1);	
  board_out(Board.regs->AUXCR, AUX_sw7210);  // compare to TNT4882 Programmer Reference, Page 4-1
  udelay(1);
  board_out(Board.regs->AUXMR, AUX_sw7210);
	udelay(1);		

	/* turn to one-chip mode */			
  Board.mirr->hssel |= HSSEL_NODMA | HSSEL_ONEC;			
	board_out( Board.regs->HSSEL, Board.mirr->hssel );			
	udelay(1);	
	
	/* reset 7210 chip and assert the local pon message */
	board_out( Board.regs->AUXMR, AUX_ch_rst );// Chip Reset
	udelay(1);		

	Board.mirr->misc = 0;		                   // affected by chip reset
	Board.mirr->hier = 0;	
	Board.mirr->intr = 0;	
	Board.mirr->accwr = 0;			
	Board.mirr->pt1  = 0;	
	Board.mirr->keyreg = 0;
	
	//board_out( Board.regs->MISC, Board.mirr->misc );		
			
	/* disable all interrupts */
  Board.mirr->imr0 = 0x80;	                 // clear all interrupt masks
	board_out( Board.regs->IMR0, Board.mirr->imr0 );	
	Board.mirr->imr1 = 0;	
	board_out( Board.regs->IMR1, Board.mirr->imr1 );	
	Board.mirr->imr2 = 0;	
	board_out( Board.regs->IMR2, Board.mirr->imr2 );
  Board.mirr->imr3 = 0;	
	board_out( Board.regs->IMR3, Board.mirr->imr3 );	
	
  Board.mirr->spmr = 0x00;                  // write initial serial response byte
  board_out(Board.regs->SPMR, Board.mirr->spmr);

  /* clear registers by reading */
  board_in( Board.regs->CPTR );         // read GPIB DIO lines

  /* read the interrupt status registers */
  board_in( Board.regs->ISR0 );         // read ISR0
  board_in( Board.regs->ISR1 );         // read ISR1
  board_in( Board.regs->ISR2 );         // read ISR2

  /* parallel poll */
	Board.mirr->ppr  = PPR;	
  Board.mirr->ppr |= PPR_U;             // unconfigure parallel poll
  board_out(Board.regs->AUXMR, Board.mirr->ppr);
  udelay(1);

	Board.mirr->admr  = 0x30;	
	Board.mirr->admr |= ADMR_ADM0;		   	
	board_out( Board.regs->ADMR, Board.mirr->admr );		

  board_out(Board.regs->CMDR, CLRSC);	      // Clear System Control
	
  /* initialize the hidden registers behind AUXMR */
	Board.mirr->auxra = AUXRA;
	Board.mirr->auxrb = AUXRB;
	Board.mirr->auxre = AUXRE;
	Board.mirr->auxrf	= AUXRF;
	Board.mirr->auxrg = AUXRG;
	Board.mirr->auxri = AUXRI;
	Board.mirr->auxrj = AUXRJ;	

  Board.mirr->auxrb |= AUXRB_CPT_ENABLE | AUXRB_TRI ;      // enable command pass through, T1 delay	(500ns)	
  board_out(Board.regs->AUXMR, Board.mirr->auxrb);		
  udelay( 1 );	

  Board.mirr->auxri |= AUXRI_PP2 | AUXRI_USTD;	// ignore remote GPIB commands, ultra short T1 delay (1100ns)
	board_out( Board.regs->AUXMR, Board.mirr->auxri );	
	udelay(1);	
	
	board_out( Board.regs->KEYREG, Board.mirr->keyreg |= KEYREG_MSTD );	// maximum speed T1 delay (350ns)	
	udelay(1);		

	/* No DMA yet */	
	Board.mirr->accwr &= ~ACCWR_DMAEN;	  		
	board_out( Board.regs->ACCWR, Board.mirr->accwr );	

  /* set the primary address */
	board_out( Board.regs->ADR, Board.pad & 0x1F );	
	//if ( Board.sad >= 0x60 && Board.sad <= 0x7E )	
		//board_out( Board.regs->ADR, ADR_ARS | ( Board.sad & 0x1F ) );	
	//else	
		//board_out( Board.regs->ADR, ADR_ARS | ADR_DT | ADR_DL );	
  board_out( Board.regs->ADR, 0xE0);   //disable ADR1

  /* enable interrupt tristate driver */
  Board.mirr->intr |= INTR_INTEN;
  board_out(Board.regs->INTR, Board.mirr->intr);

	/* Finally release pon state that was entered due to the chip reset */	
  board_out(Board.regs->AUXMR, AUX_pon);   // write pon aux. command
	udelay(1);	

	// do not halt the transfer on STBQ, ATN, SRQ and TO interrupts
  board_out(Board.regs->MISC, Board.mirr->misc |= MISC_NOAS | MISC_NOTS);

  // set FIFO Mode
  board_out(Board.regs->CFG, Board.mirr->cfg &= ~CFG_16_8N);  // 16-Byte FIFO

  // generate EOI on last byte of GPIB write, TLCINT causes a transfer to halt
  board_out(Board.regs->CFG, Board.mirr->cfg |= CFG_CCEN | CFG_TLCHLTE);

  // Enable interrupt bits
  board_out(Board.regs->IMR1, Board.mirr->imr1 |= IMR1_END_IE);	   // END received
  board_out(Board.regs->IMR2, Board.mirr->imr2 |= IMR2_ADSC_IE);	 // Adress Change
	board_out(Board.regs->IMR3,Board.mirr->imr3 |= IMR3_TLCINT_IE);  // TLCINT creates hardware interrupt, if one of the ISR0-ISR2 interrupts asserts
	
	// BTO: restart timer on each gpib transfer prevents loosing chars on recv
	board_out(Board.regs->IMR0, Board.mirr->imr0 |= IMR0_TO_IE | IMR0_BTO);
  board_out(Board.regs->AUXMR, Board.mirr->auxrj = TIMEOUT);

  // use no DMA
  board_out(Board.regs->IMR2,Board.mirr->imr2 &= ~(IMR2_DMAO | IMR2_DMAI));

  hw_count = 0;
  hw_count = -hw_count;
  board_out( Board.regs->CNT0,( unsigned char ) ( hw_count & 0xFF ) );
  board_out( Board.regs->CNT1,( unsigned char ) ( ( hw_count >>  8 ) & 0xFF ) );
  board_out( Board.regs->CNT2,( unsigned char ) ( ( hw_count >> 16 ) & 0xFF ) );
	board_out( Board.regs->CNT3,( unsigned char ) ( ( hw_count >> 24 ) & 0xFF ) );

  return 0;
}


void  release_gpib_board(dev_link_t *link)
{
	printk("release gpib board");
	
	/* disable all interrupts */
  Board.mirr->imr0 = 0x80;	                 // clear all interrupt masks
	board_out( Board.regs->IMR0, Board.mirr->imr0 );	
	Board.mirr->imr1 = 0;	
	board_out( Board.regs->IMR1, Board.mirr->imr1 );	
	Board.mirr->imr2 = 0;	
	board_out( Board.regs->IMR2, Board.mirr->imr2 );
  Board.mirr->imr3 = 0;	
	board_out( Board.regs->IMR3, Board.mirr->imr3 );	
	udelay(1);		
	
  /* disable interrupt tristate driver */
  Board.mirr->intr &= ~INTR_INTEN;
  board_out(Board.regs->INTR, Board.mirr->intr);

  free_irq(link->irq.AssignedIRQ, &Board);
}
