/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : Mon Jan 22 15:43:38 CET 2001
    copyright            : (C) 2001 by ag
    email                : arno.geissel@hbm.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


 //#include <linux/version.h>
 #include <linux/module.h>
 #include <linux/kernel.h>
 #include <linux/init.h>

 #if CONFIG_MODVERSIONS==1
 #define MODVERSIONS
 #include <linux/modversions.h>
 #endif

 #include <asm/uaccess.h> // copy_to/from_user etc.
 #include <linux/nmi.h>   // needs to be modified from original file!
 #include <linux/delay.h>
 #include <linux/slab.h>
 #include <linux/mm.h>
 #include <linux/fs.h>
 #include <linux/config.h>
 #include <linux/pci.h>

 #include "cpxxnmi.h"
 #include "allocator.h"
 #include "irq.h"
 #include "../../cpxxapp/src/target.h"// to distinguish between CP22 and CP42
 #include "../../cpxxapp/src/ga12.h"  // to share the GA12 ram structure
 #include "../../cpxxapp/src/fifo.h"  // to share the fifo structures

 #define DRIVER_AUTOR   "Arno Geissel HBM GmbH"
 #define DRIVER_DESC    "HBM GA12 driver"
 #define DRIVER_NAME    "cpxxnmi"
 #define DRIVER_SUPPORT "HBM LRP-312x (GA12)"
 #define DRIVER_LICENSE "GPL"

 static struct file_operations fops = {
  //NULL,     /* lseek */
  read:     cpxxnmi_read,     /* read */
  //NULL,     /* write */
  //NULL,     /* readdir */
  //NULL,     /* poll */
  ioctl:    cpxxnmi_ioctl,    /* ioctl */
  mmap:     cpxxnmi_mmap,     /* mmap */
  open:     cpxxnmi_open,     /* open */
  //NULL,     /* flush */
  release:  cpxxnmi_release,  /* release */
  //NULL,     /* fsync */
  //NULL,     /* fasync */
  //NULL,     /* check_media_change */
  //NULL,     /* revalidate */
  //NULL      /* lock */
 };

 static int major = DEV_MAJOR;

 // fifo issues
 unsigned long phys_fifo_addr;
 void          *fifo_ram;
 unsigned long fifo_size;                   // size of allocated fifo memory in kbytes
 unsigned long fifo_len;                    // size of allocated fifo memory in bytes
 S_FIFOGLBL    *glbl_fifo;                  // global fifo control structure
 S_FIFOCLIENT  *client_fifo;                // fifo control for each client
 S_FIFO        *fifo_ctrl[MAX_EXT_PORTS];   // fifo control for each fifo

 INT8          *fifo_data;                  // begin of data area in realtime buffer
 UINT32        fifo_data_len;               // size of the data area

 MODULE_LICENSE(DRIVER_LICENSE);
 MODULE_AUTHOR(DRIVER_AUTOR);
 MODULE_DESCRIPTION(DRIVER_DESC);
 MODULE_SUPPORTED_DEVICE(DRIVER_SUPPORT);

 void nmi_handler(void);

 // GA12 device issues
 struct   pci_dev *ga12_dev = NULL;
 unsigned ga12_pci_base;            // physical pci base of GA12 data ram
 u8       ga12_pci_irq;             // irq line of ga12

 wait_queue_head_t cpxxnmi_wait;       // waitqueue for blocking user read call operation


 int cpxxnmi_ioctl(struct inode *inodp, struct file *filep, unsigned int cmd, unsigned long arg)
 {
   S_TGA12DATA *pxmit;
   unsigned    *pram;
   //unsigned test;

   switch(cmd)
   {
     case CPXX_IOC_GFIFOSIZE:
         // return the size of fifo memory in kByte
         return ((int)fifo_size);
         break;

     case CPXX_IOC_DISABLE_NMI:
         DISABLE_NMI;
         break;

     case CPXX_IOC_ENABLE_NMI:
         ENABLE_NMI;
         break;

     case CPXX_IOC_SGA12DATA:
         pxmit = (S_TGA12DATA*)arg;
         pram = (unsigned*)ga12_ram;
         return (copy_from_user(pram+pxmit->index, pxmit->pdata, pxmit->size));
         break;

     case CPXX_IOC_GGA12DATA:
         pxmit = (S_TGA12DATA*)arg;
         pram = (unsigned*)ga12_ram;
         return (copy_to_user(pxmit->pdata, pram+pxmit->index, pxmit->size));
         break;

     case CPXX_IOC_CPXX_BEEP:
     {
         unsigned int ticks, count;

         ticks = HZ * ((arg >> 16) & 0xffff) / 1000;
         count = ticks ? (arg & 0xffff) : 0;
         if (count)
          count = 1193180 / count;

         kd_mksound(count, ticks);
         return 0;
     }

     default:
         return -EINVAL;
   }
   return 0;
 }

ssize_t cpxxnmi_read( struct file *filep, char *buf, size_t length, loff_t *wtf )
{
  interruptible_sleep_on_timeout(&cpxxnmi_wait, (long) length); // sleep max length jiffies
  //interruptible_sleep_on(&cpxxnmi_wait); // sleep
  return 0;
}


 int cpxxnmi_open(struct inode *inodp, struct file *filp)
 {
   PDEBUG("open methode called\n");
   init_waitqueue_head (&cpxxnmi_wait);
   MOD_INC_USE_COUNT;
   return 0;
 }

 int cpxxnmi_release(struct inode *inodp, struct file *filp)
 {
   // this does not work with beeping used by pcmcia_cs! ga12_ram->STR2 &= ~STR2_PAGEINT_ON;  // disable pageint
   PDEBUG("close methode called\n");
   MOD_DEC_USE_COUNT;
   return 0;
 }

 int cpxxnmi_mmap(struct file *filp, struct vm_area_struct *vma)
 {
   unsigned long start = (unsigned long)vma->vm_start;
   unsigned long size = (unsigned long)(vma->vm_end - vma->vm_start);
   int           protect = (int)vma->vm_page_prot.pgprot;

   /* int remap_page_range(virt_addr, phys_addr, size, protection); */
   if (size == GA12_DATA_RAM_SIZE)
   { // user wants to remap the GA12 PCI address space
     PDEBUG("remap_page_range(%lx, %x, %li, %i)\n", start, ga12_pci_base, size, protect);
     if (remap_page_range(start, ga12_pci_base, size, vma->vm_page_prot))
       return -EAGAIN;
   }
   else if (size == fifo_len)
   { // this only can be the fifo buffer
     PDEBUG("remap_page_range(%lx, %lx, %li, %i)\n", start, phys_fifo_addr, size, protect);
     if (remap_page_range(start, phys_fifo_addr, size, vma->vm_page_prot))
       return -EAGAIN;
   }
   else
   {
     PDEBUG("trial of refusing mmap() of %li bytes\n", size);
     return -EAGAIN;
   }

   return 0;
 }

 int ga12_init_dev(struct pci_dev *dev, int *base0, u8 *irq)
 {
   int retcode;

   if ((retcode=pci_read_config_dword(dev, PCI_BASE_ADDRESS_0, base0)) == 0)
   {
     pci_enable_device(dev);
     pci_set_master(dev);
     pci_read_config_byte(dev, PCI_INTERRUPT_LINE, irq);
   }
   return retcode;
 }

 int init_module(void)
 {
   int retcode, i;
  #ifdef CP22
   UINT16 wtemp;
   UINT8  byte;
  #else
   INT8 byte;
   struct pci_dev *dev = NULL;
  #endif

   // find GA12 on pci bus and initialize it
   if (!pci_present())
   {
     PDEBUG("no PCI bios present! device cannot be accessed ;-(\n");
     return -EIO;
   }

  #ifdef CP42
   if ((dev=pci_find_device(ALI_VENDOR, M1533_DEVICE, dev)))
   {
     if((retcode = pci_read_config_byte(dev, IORC_REGISTER, &byte)) == 0)
     {
       PDEBUG("found M1533 I/O Recovery Control %x\n", byte);

       #if 1 //
       byte = 0x03;
       if((retcode = pci_write_config_byte(dev, IORC_REGISTER, byte)) == 0)
         PDEBUG("changed M1533 Recovery Control to %x\n", byte);
       #else
       PDEBUG("did not change M1533 Recovery Control (USE_GPIB)\n");
       #endif
     }
   }
  #endif

  #ifdef CP22
  {
   UINT32 F3BAR0;
   UINT32 F0BAR0;
   UINT32 IoCtrl1;
   UINT32 IntEnable0, CfgData;
   UINT32 PinCfgAccReg;


   outl(0x80009310, 0x0CF8);     // Uhrenquarz an GPIB0
   F3BAR0 = inl(0x0CFC);
   F3BAR0 &= 0x0000FFFC;
   IoCtrl1 = inl(F3BAR0);
   IoCtrl1 |= 0x00400000;
   outl(IoCtrl1, F3BAR0);

   outl(0x80009010, 0x0CF8);
   F0BAR0 = inl(0x0CFC);
   F0BAR0 &= 0x0000FFFC;
   IntEnable0 = F0BAR0 + 0x8;
   outl(0, IntEnable0);

   PinCfgAccReg = F0BAR0 + 0x20;
   outl(0, PinCfgAccReg);

   PinCfgAccReg = F0BAR0 + 0x24;
   CfgData = inl(PinCfgAccReg);
   CfgData &= 0xFFFFFFF8;
   CfgData |= 0x03;
   outl(CfgData, PinCfgAccReg);

  }

   // Read previously set ISA CLK and I/O Recovery Control Register:
   outl(0x80009050, 0x0CF8);
   wtemp = inw(0x0CFC);
   printk("MachZ - ISA I/O Recovery Control Register, read: 0x%02X\n", (wtemp>>8));
   printk("MachZ - ISA CLK Register, read: 0x%02X\n", (wtemp & 0xFF));

   // Set ISA CLK Divisor to "Divide by 4", Set ISA I/O Recovery Control Reg to "2 CLK":
   outl(0x80009050, 0x0CF8);  // ISA I/O Recovery Control Register
   outw( ((wtemp & 0x00FC) | 0x1103), 0x0CFC);

   // Reread and display the written values:
   outl(0x80009050, 0x0CF8);
   wtemp = inw(0x0CFC);
   printk("MachZ - ISA I/O Recovery Control Register, set to: 0x%02X\n", (wtemp>>8));
   printk("MachZ - ISA CLK Register, set to: 0x%02X\n", (wtemp & 0xFF));

   outw(0x11b, 0x24);
   i = inw(0x26);
   printk("0x11b is %0X   \n",i);
   i &= ~0x02;     // PCIM2DRM_BRST_EN off: 128 MW in 50us
   outw(i,0x26);
   i = inw(0x26);
   printk("0x11b is now %0x Burst to DRAM off\n",i);

   //outw(0x11a, 0x24);
   //i = inw(0x26);
   //printk("0x11a is %0x   \n",i);
   //i &= ~0x02;     // PCIM2DRM_BRST_EN off: 128 MW in 50us
   //outw(i,0x26);
   //i = inw(0x26);
   //printk("0x11a is now %0X Burst to DRAM off\n",i);

   outb(0xc3, 0x22); // select CCR3
   byte = inb(0x23); // read CCR3
   byte |= 0x02;     // NMI enabled during SMM
   outb(0xc3, 0x22); // select CCR3
   outb(byte, 0x23);
   outb(0xc3, 0x22); // select CCR3
   byte = inb(0x23); // read CCR3
   printk("changed CCR3 to: %02x\n", byte);

   outb(0xc1, 0x22); // select CCR1
   byte = inb(0x23); // read CCR1
   byte &= ~0x02;
   outb(0xc1, 0x22); // select CCR1
   outb(byte, 0x23);
   outb(0xc1, 0x22); // select CCR1
   byte = inb(0x23); // read CCR1
   printk("changed CCR1: %02x\n", byte);
  #endif

   if ((ga12_dev=pci_find_device(GA12_VENDOR, GA12_DEVICE, ga12_dev)))
   {
     if ((retcode=ga12_init_dev(ga12_dev, (u32*)&ga12_pci_base, &ga12_pci_irq)) != 0)
     {
       PDEBUG("Error on GA12 initialisation: %x\n", retcode);
       return -EIO;
     }
     else
       PDEBUG("found GA12. Vendor: %x, Device: %x, Base: %lx, IRQ: %d\n", GA12_VENDOR, GA12_DEVICE, (unsigned long)ga12_pci_base, ga12_pci_irq);
   }
   else
   {
     PDEBUG("cannot find GA12 device ;-(\n");
     return -EIO;
   }
   // ioremap GA12 Data Ram not cachable
   if ((ga12_ram = (S_GA12_RAM*)ioremap_nocache((ga12_pci_base), GA12_DATA_RAM_SIZE)) == 0)
   {
     PDEBUG("error remapping GA12 data ram ;-(\n");
     return -EIO;
   }
   else
     PDEBUG("remapped %i bytes of GA12 data ram from %lx to %p\n", GA12_DATA_RAM_SIZE, (unsigned long)(ga12_pci_base), ga12_ram);

   PDEBUG("at first GA12 address %lx is found\n", *(unsigned long volatile *)ga12_ram);

   // allocate DMA suitable memory above high_memory;
   if (allocator_init())
   {
     PDEBUG("allocator init failed, no fifo memory available, NMI handler not loaded :-(\n");
     return -EIO;
   }

   if ((phys_fifo_addr = allocator_allocate_dma(fifo_size, GFP_KERNEL)) != 0)
   {
     PDEBUG("%li bytes allocated for FIFO at adress %lx and %p\n", fifo_size << 10, phys_fifo_addr, phys_to_virt(phys_fifo_addr));
     fifo_len = fifo_size << 10;

     if( (fifo_ram = ioremap_nocache(__pa(high_memory), fifo_len)) == NULL )
     {
       PDEBUG("remapping of %li bytes for FIFO failed.. goodbye world :-(\n", fifo_len);
       return -EIO;
     }

     glbl_fifo = (S_FIFOGLBL *)fifo_ram;           // global fifo structure is at the beginning of fifo_ram
     PDEBUG("glbl_fifo at: %p\n", glbl_fifo);

     client_fifo = (S_FIFOCLIENT *)(glbl_fifo + 1);// initialize client_fifo pointer field
     PDEBUG("client_fifo at: %p\n", client_fifo);

     fifo_ctrl[0] = (S_FIFO *)(client_fifo + MAX_EXT_PORTS); // initialize fifo_ctrl pointer field
     for(i=1; i<MAX_EXT_PORTS; i++)
     {
       fifo_ctrl[i] = fifo_ctrl[i-1] + MAX_FIFO;
       PDEBUG("fifo_ctrl[%i]: %p\n", i, fifo_ctrl[i]);
     }

     fifo_data = (INT8 *)(fifo_ctrl[i-1] + MAX_FIFO);  // initialize fifo_data pointer
     fifo_data_len = fifo_len - (fifo_data - (INT8*)glbl_fifo);
     PDEBUG("fifo_data: %p ; len: %ld \n", fifo_data, fifo_data_len);
     ga12_ram->DMA_WrDestOfs = phys_fifo_addr + (fifo_data-(INT8*)fifo_ram); // the destination offset: begin of realtime buffer
   }
   else
   {
     PDEBUG("allocation of %li bytes for FIFO failed.. goodbye world :-(\n", fifo_len);
     return -EIO;
   }

   // install special NMI handler. This needs a patched kernel!
   PDEBUG("Default NMI handler loading...\n");
   linux_nmi_hook = (nmi_hook *)&nmi_handler;
   PDEBUG("NMI hook is set to %p\n", nmi_handler);

   // register as chacter device
   if ( (retcode = register_chrdev(major, MODULE_NAME, &fops)) != 0 )
   {
     PDEBUG("register_chrdev failed.. goodbye world :-(\n");
     return -EIO;
   }

   // get an interrupt (IRQ)
   register_irq();
   //ENABLE_NMI;

   // print the adress of the first .text symbol
   PDEBUG("module starts at: %p\n", cpxxnmi_ioctl);

   return 0;
 }

 void cleanup_module(void)
 {
   // disable interrupts
   ga12_ram->STR2 &= ~STR2_PAGEINT_ON;  // disable pageint
   DISABLE_NMI;

   // deinstall special handler
   linux_nmi_hook = (nmi_hook *)NULL;

   PDEBUG("freeing GA12 data ram at %p\n", ga12_ram);

   iounmap((void *)ga12_ram);

   allocator_free_dma(phys_fifo_addr);
   allocator_cleanup();

   // deregister interrupt (IRQ)
   cleanup_irq();

   if (unregister_chrdev(major, MODULE_NAME) != 0)
     PDEBUG("cleanup_module failed\n");
   else
     PDEBUG("module unloaded\n");
 }


static void kd_nosound(unsigned long ignored)
{
  /* disable counter 2 */
  outb(inb_p(0x61)&0xFC, 0x61);
  return;
}

void kd_mksound(unsigned int hz, unsigned int ticks)
{
  static struct timer_list sound_timer = { function: kd_nosound };
  unsigned int count = 0;
  unsigned long flags;

  if (hz > 20 && hz < 32767)
    count = 1193180 / hz;

#ifdef CONFIG_SMP
#error
#endif

  save_flags(flags);
  cli();
  del_timer(&sound_timer);

  if (count)
  {
    /* enable counter 2 */
    outb_p(inb_p(0x61)|3, 0x61);
    /* set command for counter 2, 2 byte write */
    outb_p(0xB6, 0x43);
    /* select desired HZ */
    outb_p(count & 0xff, 0x42);
    outb((count >> 8) & 0xff, 0x42);

    if (ticks) {
      sound_timer.expires = jiffies+ticks;
      add_timer(&sound_timer);
    }
  }
  else
    kd_nosound(0);

  restore_flags(flags);
  return;
}





