/***************************************************************************
                          irq.c  -  description
                             -------------------
    begin                : Wed Sep 11 2002
    copyright            : (C) 2002 by ag
    email                : arno.geissel@hbm.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <asm/io.h>
#include <linux/sched.h>
#include "irq.h"
#include "cpxxnmi.h"

#include "../../cpxxapp/src/target.h"//
#include "../../cpxxapp/src/ga12.h"  // to share the GA12 ram structure
#include "../../cpxxapp/src/fifo.h"  // to share the fifo structures

#ifdef USE_RTNET
 #include <rtai.h>
 #include <rtai_sched.h>
#endif

const UINT32 INT_STAT[] = {
                           GA12_INT_ROLL,
                           GA12_INT_TICTAC,
                           GA12_INT_TEVT,
                           GA12_INT_HOME,
                           0
                         };

#define L_INT_STAT  sizeof(INT_STAT)

#ifdef USE_RTNET

static void ga12_interrupt(unsigned int irq)
{
  UINT32                *pint_stat;
  INT32                 dev_stat;
  S_FIFOCLIENT          *pclient_fifo;
  INT16                 client;
  //struct task_struct    *p;
  #ifdef SHOWBIT_DEBUG
  //UINT8                 btemp;
  #endif

  #ifdef SHOWBIT_DEBUG
  //btemp = inb(0x378) | 0x02;
  //outb(btemp, 0x378);
  #endif

  dev_stat = ga12_ram->IntRes;  // get status of GA12 and reset all interrupt flags
  if(!(dev_stat & (GA12_INT_ROLL | GA12_INT_TICTAC | GA12_INT_TEVT | GA12_INT_HOME)))
  {
    goto ret;                   // checked them all together: not for us!
  }
  ga12_ram->HINT = 0;           // reset home interrupt flag


  pint_stat = (INT32*)&INT_STAT[0];

  rt_disable_irq(irq);
  //rt_mask_and_ack_irq(irq);

  while(*pint_stat)
  {
    if(dev_stat & *pint_stat)
    {
      switch(*pint_stat)
      {
        case GA12_INT_ROLL:
          break;

        case GA12_INT_TICTAC:
          break;

        case GA12_INT_TEVT:
          break;

        case GA12_INT_HOME:
          for(client=0; client<MAX_EXT_PORTS; client++)
          {
            //if(client_fifo[client].active)
            {
              pclient_fifo = client_fifo + client;

              if(pclient_fifo->rt_pid)
              {
                if(pclient_fifo->rt_pid & 0xc0000000)  // its a LXRT thread
                {
                  pclient_fifo->rt_pidsave = pclient_fifo->rt_pid;
                  pclient_fifo->rt_pid = 0;            // no new resumes until task has finished

                  //if(rt_get_task_state((RT_TASK*)pclient_fifo->rt_pidsave) & SUSPENDED)
                  {
                    rt_sched_lock();

                    rt_task_resume((RT_TASK*)pclient_fifo->rt_pidsave);
                    //rt_send_if((RT_TASK*)pclient_fifo->rt_pid, 1);
                    //rt_sem_signal(pclient_fifo->rt_signal);

                    rt_sched_unlock();
                  }
                }
                else
                {
                  pclient_fifo->rt_pidsave = pclient_fifo->rt_pid;
                  pclient_fifo->rt_pid = 0;            // no new resumes until task has finished

                  wake_up_process(find_task_by_pid(pclient_fifo->rt_pidsave));
                }
              }
            }
          } // for
          break;
      }
    }
    pint_stat++;
  }

  #ifdef SHOWBIT_DEBUG
  //outb(btemp & ~2, 0x378);
  #endif

  rt_enable_irq(irq);
  //rt_unmask_irq(irq);

  ret:
  return;
}

#else  //USE_RTNET

void ga12_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
  UINT32                *pint_stat;
  INT32                 dev_stat;
  #ifdef SHOWBIT_DEBUG
  UINT8                 btemp;
  #endif

  unsigned long         flags;

  save_flags(flags);

  if(dev_id != ga12_dev)  // an other source generated the interrupt
  {
    goto ret;
  }

  #ifdef SHOWBIT_DEBUG
  btemp = inb(0x378) | 0x02;
  outb(btemp, 0x378);
  #endif

  dev_stat = ga12_ram->IntRes;  // get status of GA12 and reset all interrupt flags
  if(!(dev_stat & (GA12_INT_ROLL | GA12_INT_TICTAC | GA12_INT_TEVT | GA12_INT_HOME)))
  {
    goto ret;                   // checked them all together: not for us!
  }
  ga12_ram->HINT = 0;           // reset home interrupt flag


  pint_stat = (INT32*)&INT_STAT[0];

  while(*pint_stat)
  {
    if(dev_stat & *pint_stat)
    {
      switch(*pint_stat)
      {
        case GA12_INT_ROLL:
          break;

        case GA12_INT_TICTAC:
          break;

        case GA12_INT_TEVT:
          break;

        case GA12_INT_HOME:
          wake_up_interruptible(&cpxxnmi_wait);
          break;
      }
    }
    pint_stat++;
  }

  #ifdef SHOWBIT_DEBUG
  btemp = inb(0x378) & ~0x02;
  outb(btemp, 0x378);
  #endif

  ret:
  restore_flags(flags);
  return;
}
#endif //USE_RTNET



int register_irq(void)
{
//  SET_INTERRUPT_EDGE;
//  SET_INTERRUPT_LEVEL;

  // request interrupt:

 #ifdef USE_RTNET
  if (rt_request_global_irq(ga12_pci_irq, (void (*)(void))ga12_interrupt) != 0)
  {
    PDEBUG("rt_request_global_irq for IRQ %d failed.\n", ga12_pci_irq);
    return(-1);
  }
  else
  {
    rt_enable_irq(ga12_pci_irq);
    PDEBUG("realtime IRQ %d installed.\n", ga12_pci_irq);
  }
 #else
  if (request_irq(ga12_pci_irq, ga12_interrupt, SA_INTERRUPT, MODULE_NAME, ga12_dev) != 0 )
  {
    if (request_irq(ga12_pci_irq, ga12_interrupt, SA_SHIRQ, MODULE_NAME, ga12_dev) != 0 )
    {
      PDEBUG("request_irq for IRQ %d failed.\n", ga12_pci_irq);
      return(-1);
    }
    else
      PDEBUG("shared IRQ %d installed.\n", ga12_pci_irq);
  }
  else
  {
    PDEBUG("nonshared IRQ %d installed.\n", ga12_pci_irq);
  }
 #endif

  return(0);
}


int cleanup_irq(void)
{
  // free interrupt
 #ifdef USE_RTNET
  rt_disable_irq(ga12_pci_irq);
  rt_free_global_irq(ga12_pci_irq);
 #else
  free_irq(ga12_pci_irq, ga12_dev);
 #endif
  return(0);
}
