// JDrvOsLi.c ansi_c&linux          
// JIDA driver low level Linux functions
// {G)U(2} 2001.10.11 2004.06.11

/****************************************************************************/

#include <linux/version.h>
#include <linux/malloc.h>

#include <linux/delay.h> 
#include <asm/io.h> 
#include <wchar.h> 

typedef unsigned char uchar; 

#include "JWinDefs.h"
#include "JDrvOs.h"

/****************************************************************************/

/* ioremap */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,1,0))
#if defined(_LINUX_MM_H)
#  define ioremap_nocache ioremap
#  ifndef __i386__
   /* This simple approach works for non-PC platforms. */
#    define ioremap vremap
#    define iounmap vfree
#  else /* the PC has <expletive> ISA; 2.2 and 2.4 remap it, 2.0 needs not */
extern inline void *ioremap(unsigned long phys_addr, unsigned long size)
{
    if (phys_addr >= 0xA0000 && phys_addr + size <= 0x100000)
        return (void *)phys_addr;
    return vremap(phys_addr, size);
}

extern inline void iounmap(void *addr)
{
    if ((unsigned long)addr >= 0xA0000
            && (unsigned long)addr < 0x100000)
        return;
    vfree(addr);
}
#  endif
# endif
#endif

/****************************************************************************/
// Standard port IO

uchar JosReadPort8(uint port)
  {
  return inb(port);
  }
  
ushort JosReadPort16(uint port)
  {
  return inw(port);
  }
  
ulong JosReadPort32(uint port)
  {
  return inl(port);
  }

void JosWritePort8(uint port, uint value)
  {
  outb(value,port);
  }

void JosWritePort16(uint port, uint value)
  {
  outw(value,port);
  }

void JosWritePort32(uint port, ulong value)
  {
  outl(value,port);
  }

/****************************************************************************/
// Delay functions

void JosSleepms(ulong ms)  
  { 
#if (LINUX_VERSION_CODE>=KERNEL_VERSION(2,2,0))
//  __set_current_state(TASK_UNINTERRUPTIBLE);
  current->state=TASK_INTERRUPTIBLE;
  schedule_timeout((ms*HZ+999)/1000); // timeout value in jiffies
#else
  current->state=TASK_INTERRUPTIBLE;
  current->timeout=jiffies+((ms*HZ+999)/1000);
  schedule();
  current->state=TASK_RUNNING; // FIXME: do we need that?
#endif
  }

void JosWaitus(ulong us)
  {
   udelay(us);
  }

/****************************************************************************/
// Memory allocation and mapping

void *JosAllocMem(ulong len)
  {
  return kmalloc(len,GFP_KERNEL);
  }

void JosFreeMem(void *p)
  {
  kfree(p);
  }

/****************************************************************************/

void *JosMapAddress(ulong AddrHigh, ulong AddrLow, ulong AddressSpace, ulong NumberOfBytes)
  {
	return ioremap(AddrLow, NumberOfBytes);
  }

void JosUnMapAddress(void *base, ulong type, ulong len)
  {
	iounmap(base);
  }

/****************************************************************************/
