// linuxif.c ansi_c&linux
// JIDA Linux driver
// {G)U(2} 2001.11.06 2004.06.11

// $Id: linuxif.c,v 1.4 2003/09/09 15:13:56 michael Exp $

/****************************************************************************/ 

#include <linux/version.h>
#define __NO_VERSION__  // don't define the kernel version global variable
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/malloc.h>

#include <errno.h>

#include "JDrvUp.h"

/****************************************************************************/ 
/* Definitions concerning DEVFS */
#ifdef CONFIG_DEVFS_FS
#include <linux/devfs_fs_kernel.h>
devfs_handle_t jida_devfs_handle;
#else
typedef void * devfs_handle_t;
#endif

#ifndef DEVFS_FL_DEFAULT
extern inline void devfs_unregister(devfs_handle_t de) {}
#endif

/****************************************************************************/ 

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0))
#include <linux/slab.h>
#include <asm/uaccess.h>
#else
#include <asm/segment.h>
#endif

/****************************************************************************/ 

struct IOControlPacket { 
	unsigned char *pInBuffer; 
    	unsigned long InBufferSize; 
    	unsigned char * pOutBuffer; 
    	unsigned long OutBufferSize; 
	unsigned long *pBytesReturned; 
}; 

/****************************************************************************/ 

// major number of the jida driver
#define MAJOR_NO 99

// UNIX device name
#define UNIX_DEVICE_NAME "/dev/jida"

// driver local data structure
typedef struct {
  void *pJidaDeviceInfo;
  struct semaphore IoSem;
  } LOCAL_DEVICE_INFO, *PLOCAL_DEVICE_INFO;

LOCAL_DEVICE_INFO device_info;

extern struct file_operations jidafops;

int jida_init(void);
void jida_cleanup(void);
int jida_open(struct inode* _inode, struct file *f);
int jida_close(struct inode* _inode, struct file* f);
int jida_ioctl(struct inode* _inode, struct file* f, unsigned int command, unsigned long arg);

/****************************************************************************/ 

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,1,0))

int copy_from_user(void *to, void *from, int n) 
{
  if (verify_area (VERIFY_READ, from, n))
    return 1;
  memcpy_fromfs(to, from, n);
  return 0;
}

int copy_to_user(void *to, void *from, int n) 
{
  if (verify_area (VERIFY_WRITE, to, n))
    return 1;
  memcpy_tofs(to, from, n);
  return 0;
}

#endif

/****************************************************************************/ 

#ifdef MODULE   

int init_module(void)
{
  return jida_init();
}

void cleanup_module(void)
{
  jida_cleanup();
}

#endif

/****************************************************************************/ 

/* Install Driver */
int jida_init(void)
{       
  int stat = 0;
  
  printk("Install Jida-Driver...\n");

#if (LINUX_VERSION_CODE <= KERNEL_VERSION(2,1,0))
  register_symtab (NULL);
#endif
  
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0))
  sema_init(&device_info.IoSem, 1);
#else
  device_info.IoSem.count = 1;
  device_info.IoSem.waking = 0;
  device_info.IoSem.lock = 0;
  device_info.IoSem.wait = NULL;
#endif

#ifdef CONFIG_DEVFS_FS
  jida_devfs_handle = devfs_register(NULL, "jida",
		 DEVFS_FL_AUTO_DEVNUM,
		 0, 0, S_IFCHR | S_IXUGO,
		 &jidafops, NULL);
  if (jida_devfs_handle < 0) return (int) jida_devfs_handle;

#else
  stat = register_chrdev(MAJOR_NO, UNIX_DEVICE_NAME, &jidafops );
  if (stat < 0) {
    printk("Error: Could not register Jida-Driver\n");
    return(1);
  }
#endif /* CONFIG_DEVFS_FS */
             
  device_info.pJidaDeviceInfo = JduInitBoards(0);

  if (!device_info.pJidaDeviceInfo || !JduGetBoardCount(device_info.pJidaDeviceInfo))
  {
    printk("Error: Could not find a jumptec board\n");

#ifdef CONFIG_DEVFS_FS
    devfs_unregister(jida_devfs_handle);
#else
    unregister_chrdev(MAJOR_NO, UNIX_DEVICE_NAME);
#endif

    return -ENXIO;
  }
  else
  {
    int i, ii = JduGetBoardCount(device_info.pJidaDeviceInfo);
    for (i = 0; i < ii; i++)
      printk("found jumptec board: %s\n", JduGetBoardName(device_info.pJidaDeviceInfo,i));
  }
    
  printk("Jida-Driver installed\n");
  
  return 0;
}

/* Uninstall Driver */
void jida_cleanup(void)
{
  printk("Uninstall Jida-Driver...");
  
  JduUnInitBoards(device_info.pJidaDeviceInfo);

#ifndef CONFIG_DEVFS_FS
  unregister_chrdev(MAJOR_NO, UNIX_DEVICE_NAME);
#else
  devfs_unregister(jida_devfs_handle);
#endif
  
  printk("Ok\n");

}

int jida_open(struct inode* _inode, struct file *f)
{
  int ret = 0;

  MOD_INC_USE_COUNT;

  return ret;
}

int jida_close(struct inode* _inode, struct file* f)
{
  MOD_DEC_USE_COUNT;

  return 0;
}

int jida_ioctl(struct inode* _inode, struct file* f, unsigned int command, unsigned long arg)
{
  int    ret = 0;
  unsigned long  BytesReturned = 0;
  unsigned char *pOutBuffer_orig = 0;
  unsigned char * pInBuffer_orig = 0;
  
  struct IOControlPacket io_packet;

  /*printk("Enter ioctl-function with command = 0x%x\n", command);*/

  ret = copy_from_user(&io_packet, (struct IOControlPacket*) arg, 
           sizeof(struct IOControlPacket));
  if (ret)
    return -EFAULT;

  pOutBuffer_orig = io_packet.pOutBuffer;
  pInBuffer_orig = io_packet.pInBuffer;

  io_packet.pInBuffer = 0;
  if (io_packet.InBufferSize > 0) {
    io_packet.pInBuffer = kmalloc (io_packet.InBufferSize, GFP_KERNEL);

    if (!io_packet.pInBuffer)
      return -ENOMEM;
    ret = copy_from_user (io_packet.pInBuffer, 
        pInBuffer_orig,
        io_packet.InBufferSize);
    if (ret)
      return -EFAULT;
  }
  
  io_packet.pOutBuffer = NULL;
  if (io_packet.OutBufferSize > 0) {
    (void*)io_packet.pOutBuffer = kmalloc(io_packet.OutBufferSize, 
                  GFP_KERNEL);
    if (io_packet.pOutBuffer == NULL) 
    {
      kfree(io_packet.pInBuffer); 
      return -ENOMEM;
    }
  }
  
  /* Enter CriticalSection */
  down(&device_info.IoSem);

  ret = JduIOControl(device_info.pJidaDeviceInfo, command, 
           (unsigned long *) io_packet.pInBuffer, 
           io_packet.InBufferSize, 
           (unsigned long *) io_packet.pOutBuffer, 
           io_packet.OutBufferSize, &BytesReturned);

  /* Leave CriticalSection */
  up(&device_info.IoSem);

  if (BytesReturned > 0)
  {
    ret = copy_to_user(pOutBuffer_orig, io_packet.pOutBuffer, 
           BytesReturned);
    if (ret)
      return -EFAULT;

    ret = copy_to_user(io_packet.pBytesReturned, &BytesReturned, 
           sizeof(unsigned long));
    if (ret)
      return -EFAULT;
  }
  
  if ((io_packet.pInBuffer != NULL))
    kfree(io_packet.pInBuffer); 
  if (io_packet.pOutBuffer != NULL)
    kfree(io_packet.pOutBuffer);

  /*printk("Leave ioctl-function with command = 0x%x\n", command);*/
  
  return (-ret);
}

/****************************************************************************/ 

struct file_operations jidafops = {
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0))
    	owner:		THIS_MODULE,
	ioctl:		jida_ioctl,
	open:		jida_open,
	release:	jida_close,
#else
    NULL,        /* lseek */ 
    NULL,        /* read */
    NULL,        /* write */
    NULL,        /* readdir */
    NULL,        /* poll */
    &jida_ioctl, /* ioctl */
    NULL,        /* mmap */
    &jida_open,  /* open */
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0))
    NULL,        /* flush */
    &jida_close, /* release */
#else
    (void (*) (struct inode *, struct file *))&jida_close, /* release */
#endif
#endif
};         

/****************************************************************************/ 

