/* Treiber fr LED-Flasher auf ARM-Board */
/* $Id: led_module.c,v 1.1.1.1 2001/12/13 13:09:21 kanne Exp $ */


#include <asm/uaccess.h>
#include <asm/arch/hardware.h>	/*Adressmapping*/

#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <linux/init.h>

#include <asm/io.h>
#include <asm/hardware/ep7212.h>


static int 
led_open(struct inode *inode, struct file *file)
{
  return 0;
}


static int
led_release(struct inode *inode, struct file *file)
{
  return 0;
}


static ssize_t
led_write( struct file *file, const char *data, size_t len, loff_t *ppos )
{
  ssize_t nread = 0;

  /*  Can't seek (pwrite) on this device  */
  if ( ppos != &file->f_pos )
       return -ESPIPE;
	
  if ( len )
     {
       char buffer[1];

       if ( copy_from_user( buffer, data, 1 ) )
	    return -EFAULT;

       nread = 1;

       //  printk("<1>Daten des write-Befehls: 0x%.2x\n", buffer[0]);
       clps_writeb( buffer[0], LEDFLSH );
     }

  return nread;
}


static struct file_operations led_fops=
{
  owner:	THIS_MODULE,
  write:	led_write,
  open:		led_open,
  release:	led_release,
};


static struct miscdevice led_miscdev=
{
  LED_MINOR,
  "led",
  &led_fops
};


int __init 
led_init( void )
{
  int rv = misc_register( &led_miscdev );
  
  if ( rv )
     {
       printk( KERN_ERR "can't register led device !" );
       return rv;
     }

  printk( "led device driver loaded.\n" );
  clps_writeb( 0x7f, LEDFLSH );

  return 0;
}


void __exit 
led_exit( void )
{
  clps_writeb( 0, LEDFLSH );	/* 0111111 */

  misc_deregister( &led_miscdev );
  printk( "led device driver unloaded.\n" );
}


module_init( led_init );
module_exit( led_exit );
