/* 020222 asm-arm/arch-sa1100/timer.h - ARM/SA1100 specific timer
Don't include directly - it's included through asm-arm/rtai.h

COPYRIGHT (C) 2002 Guennadi Liakhovetski, DSA GmbH (gl@dsa-ac.de)
COPYRIGHT (C) 2002 Wolfgang Mller (wolfgang.mueller@dsa-ac.de)
Copyright (c) 2001 Alex Zpke, SYSGO RTS GmbH (azu@sysgo.de)

This program is free software; you can redistribute it and/or modify
it under the terms of version 2 of the GNU General Public License as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*
--------------------------------------------------------------------------
Acknowledgements
- Paolo Mantegazza	(mantegazza@aero.polimi.it)
	creator of RTAI 
*/

#ifndef _ASM_ARCH_RTAI_TIMER_H_
#define _ASM_ARCH_RTAI_TIMER_H_

#include <asm/proc/hard_system.h>
#include <linux/time.h>
#include <linux/timex.h>

#define CPU_FREQ (tuned.cpu_freq)

union rtai_tsc {
	unsigned long long tsc;
	unsigned long hltsc[2];
};
extern union rtai_tsc rtai_tsc;

static inline RTIME rdtsc(void)
{
	RTIME ts;
	unsigned long flags, count;

	hard_save_flags_and_cli(flags);

	if ( ( count = OSCR ) < rtai_tsc.hltsc[0] )
		rtai_tsc.hltsc[1]++;
	rtai_tsc.hltsc[0] = count;
	ts = rtai_tsc.tsc;
	hard_restore_flags(flags);

	return ts;
}

#if DEBUG_LEVEL > 0
#ifdef ALLOW_RR
static int rtai_sched_path = 0;
static unsigned long next_match, _oscr;
#endif
extern unsigned long rtai_timer_crash, rtai_big_delay;
#define MIN_DT llimd(1000, FREQ_SYS_CLK, 1000000000)
#endif

#define PROTECT_TIMER

#if 1
#define REG_BASE  0xfa000000
#define REG_OSMR0 0x00000000
#define REG_OSCR  0x00000010
#define REG_OSSR  0x00000014
#define REG_ICMR  0x00050004
static inline void rt_set_timer_match_reg(int delay)
{
#if DEBUG_LEVEL == 0 || ! defined(ALLOW_RR)
	unsigned long next_match;
#endif
	unsigned long flags;

	hard_save_flags_cli( flags );

	__asm__ __volatile__ (
	"mov	r2, #0xfa000000		@ REG_BASE
	mov	r0, #1			@ OSSR_M0
 	str	r0, [r2, #0x14]		@ OSSR = OSSR_M0;\n"
#ifdef PROTECT_TIMER
"	cmp	%1, %3			@ if ( delay > LATCH || delay < 0 )
	movhi	%1, %3			@	delay = LATCH;\n"
#endif
"	teq	%1, #0			@ if ( delay == 0 ) {
	ldreq	r1, [r2, #0]		@	r1 = OSMR0;
	addeq	r0, r1, %2		@	r0 = r1 + rt_times.periodic_tick;
	ldrne	r0, [r2, #0x10]		@ } else { r0 = OSCR;
	addne	r0, r0, %1		@	r0 = r0 + delay; }
	str	r0, [r2, #0]		@ OSMR0 = r0;\n"
#if DEBUG_LEVEL > 0
"	mov	%0, r0			@ next_match = r0;\n"
#endif
"	add	r2, r2, #0x50000
	ldr	r0, [r2, #0x4]		@ unmask OSMR0 irq
	orr	r0, r0, #04000000	@ (1 << 26)
	str	r0, [r2, #0x4]"
		: "=r" (next_match)
		: "r" (delay), "r" (rt_times.periodic_tick), "r" (LATCH)
		: "r0", "r1", "r2", "memory", "cc" );

	hard_restore_flags( flags );
}

#else
/* Current version */
static inline void rt_set_timer_match_reg(int delay)
{
	extern union rtai_tsc rtai_tsc;
	unsigned long flags;
#if DEBUG_LEVEL == 0
	unsigned long next_match;
#else
	int count = 0;
#endif

#ifdef PROTECT_TIMER
	if ( delay > LATCH )
		delay = LATCH;
#endif

	hard_save_flags_cli(flags);
	OSSR = OSSR_M0;

	if ( delay )
		next_match = OSMR0 = delay + OSCR;
//		next_match = OSMR0 = delay + rtai_tsc.hltsc[0];
	else
		next_match = ( OSMR0 += rt_times.periodic_tick );
//		next_match = OSMR0 = ((unsigned long *)&rt_times.intr_time)[0];

#ifdef PROTECT_TIMER
	while ((int)(next_match - OSCR) < LATENCY_TICKS ) {
#if DEBUG_LEVEL > 0
		count++;
#endif
		next_match = OSMR0 = OSCR + LATENCY_TICKS;
	}
#endif

	rt_unmask_irq( TIMER_8254_IRQ );
	hard_restore_flags(flags);
}
#endif

#define rt_set_timer_delay(x)  rt_set_timer_match_reg(x)

#endif
